@@ -5,17 +5,37 @@
 //Lambertian
 //
 
-vec3 lambertian::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
+inline bool Refract(const vec3f &wi, const normal3f &n, Float eta, vec3f *wt) {
+  // Compute $\cos \theta_\roman{t}$ using Snell's law
+  if(eta == 1) {
+    *wt = -wi;
+    return(true);
+  }
+  Float cosThetaI = dot(n, wi);
+  Float sin2ThetaI = std::fmax(Float(0), Float(1 - cosThetaI * cosThetaI));
+  Float sin2ThetaT = eta * eta * sin2ThetaI;
+  
+  // Handle total internal reflection for transmission
+  if (sin2ThetaT >= 1) return(false);
+  Float cosThetaT = std::sqrt(1 - sin2ThetaT);
+  *wt = eta * -wi + (eta * cosThetaI - cosThetaT) * vec3f(n.x(),n.y(),n.z());
+  return(true);
+}
+
+
+point3f lambertian::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
   //unit_vector(scattered.direction()) == wo
   //r_in.direction() == wi
-  vec3 wi = unit_vector(scattered.direction());
-  Float cosine = dot(rec.normal, wi);
+  vec3f wi = unit_vector(scattered.direction());
+  normal3f n = unit_vector(rec.normal);
+  Float cosine = dot(n, wi);
   //Shadow terminator if bump map
   Float G = 1.0f;
   if(rec.has_bump) {
+    
     Float NsNlight = dot(rec.bump_normal, wi);
-    Float NsNg = dot(rec.bump_normal, rec.normal);
-    G = NsNlight > 0.0 && NsNg > 0.0 ? std::fmin(1.0, dot(wi, rec.normal) / (NsNlight * NsNg)) : 0;
+    Float NsNg = dot(rec.bump_normal, n);
+    G = NsNlight > 0.0 && NsNg > 0.0 ? std::fmin(1.0, dot(wi, n) / (NsNlight * NsNg)) : 0;
     G = G > 0.0f ? -G * G * G + G * G + G : 0.0f;
     cosine = dot(rec.bump_normal, wi);
   }
@@ -24,19 +44,21 @@ vec3 lambertian::f(const ray& r_in, const hit_record& rec, const ray& scattered)
   }
   return(G * albedo->value(rec.u, rec.v, rec.p) * cosine * M_1_PI);
 }
+
 bool lambertian::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, random_gen& rng) {
   srec.is_specular = false;
   srec.attenuation = albedo->value(hrec.u, hrec.v, hrec.p);
   srec.pdf_ptr = new cosine_pdf(hrec.normal);
   return(true);
 }
+
 bool lambertian::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, Sampler* sampler) {
   srec.is_specular = false;
   srec.attenuation = albedo->value(hrec.u, hrec.v, hrec.p);
   srec.pdf_ptr = new cosine_pdf(hrec.normal);
   return(true);
 }
-vec3 lambertian::get_albedo(const ray& r_in, const hit_record& rec) const {
+point3f lambertian::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
@@ -45,14 +67,14 @@ vec3 lambertian::get_albedo(const ray& r_in, const hit_record& rec) const {
 //
 
 bool metal::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, random_gen& rng) {
-  vec3 normal = !hrec.has_bump ? hrec.normal : hrec.bump_normal;
-  vec3 wi = -unit_vector(r_in.direction());
-  vec3 reflected = Reflect(wi,unit_vector(normal));
+  normal3f normal = !hrec.has_bump ? hrec.normal : hrec.bump_normal;
+  vec3f wi = -unit_vector(r_in.direction());
+  vec3f reflected = Reflect(wi,unit_vector(normal));
   Float cosine = AbsDot(unit_vector(reflected), unit_vector(normal));
   if(cosine < 0) {
     cosine = 0;
   }
-  vec3 offset_p = offset_ray(hrec.p-r_in.A, hrec.normal) + r_in.A;
+  point3f offset_p = offset_ray(hrec.p-r_in.A, hrec.normal) + r_in.A;
   
   srec.specular_ray = ray(offset_p, reflected + fuzz * rng.random_in_unit_sphere(), r_in.pri_stack, r_in.time());
   srec.attenuation = albedo->value(hrec.u, hrec.v, hrec.p) * FrCond(cosine, eta, k);
@@ -60,15 +82,16 @@ bool metal::scatter(const ray& r_in, const hit_record& hrec, scatter_record& sre
   srec.pdf_ptr = 0;
   return(true);
 }
+
 bool metal::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, Sampler* sampler) {
-  vec3 normal = !hrec.has_bump ? hrec.normal : hrec.bump_normal;
-  vec3 wi = -unit_vector(r_in.direction());
-  vec3 reflected = Reflect(wi,unit_vector(normal));
+  normal3f normal = !hrec.has_bump ? hrec.normal : hrec.bump_normal;
+  vec3f wi = -unit_vector(r_in.direction());
+  vec3f reflected = Reflect(wi,unit_vector(normal));
   Float cosine = AbsDot(unit_vector(reflected), unit_vector(normal));
   if(cosine < 0) {
     cosine = 0;
   }
-  vec3 offset_p = offset_ray(hrec.p-r_in.A, hrec.normal) + r_in.A;
+  point3f offset_p = offset_ray(hrec.p-r_in.A, hrec.normal) + r_in.A;
   
   srec.specular_ray = ray(offset_p, reflected + fuzz * rand_to_unit(sampler->Get2D()), r_in.pri_stack, r_in.time());
   srec.attenuation = albedo->value(hrec.u, hrec.v, hrec.p) * FrCond(cosine, eta, k);
@@ -76,7 +99,8 @@ bool metal::scatter(const ray& r_in, const hit_record& hrec, scatter_record& sre
   srec.pdf_ptr = 0;
   return(true);
 }
-vec3 metal::get_albedo(const ray& r_in, const hit_record& rec) const {
+
+point3f metal::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
@@ -86,10 +110,12 @@ vec3 metal::get_albedo(const ray& r_in, const hit_record& rec) const {
 
 bool dielectric::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, random_gen& rng) {
   srec.is_specular = true;
-  vec3 outward_normal;
-  vec3 normal = !hrec.has_bump ? hrec.normal : hrec.bump_normal;
+  normal3f outward_normal;
+  normal3f wh = !hrec.has_bump ? hrec.normal : hrec.bump_normal;
+  wh.make_unit_vector();
+  
+  vec3f wi = -unit_vector(r_in.direction());
   
-  vec3 reflected = reflect(r_in.direction(), normal);
   Float ni_over_nt;
   srec.attenuation = albedo;
   Float current_ref_idx = 1.0;
@@ -101,7 +127,7 @@ bool dielectric::scatter(const ray& r_in, const hit_record& hrec, scatter_record
   
   bool entering = dot(hrec.normal, r_in.direction()) < 0;
   bool skip = false;
-  vec3 offset_p = hrec.p;
+  point3f offset_p = hrec.p;
   
   for(size_t i = 0; i < r_in.pri_stack->size(); i++) {
     if(r_in.pri_stack->at(i) == this) {
@@ -124,46 +150,37 @@ bool dielectric::scatter(const ray& r_in, const hit_record& hrec, scatter_record
   if(skip) {
     srec.specular_ray = ray(offset_p, r_in.direction(), r_in.pri_stack, r_in.time());
     Float distance = (offset_p-r_in.point_at_parameter(0)).length();
-    vec3 prev_atten = r_in.pri_stack->at(prev_active)->attenuation;
-    srec.attenuation = vec3(std::exp(-distance * prev_atten.x()),
-                            std::exp(-distance * prev_atten.y()),
-                            std::exp(-distance * prev_atten.z()));
+    point3f prev_atten = r_in.pri_stack->at(prev_active)->attenuation;
+    srec.attenuation = point3f(std::exp(-distance * prev_atten.x()),
+                               std::exp(-distance * prev_atten.y()),
+                               std::exp(-distance * prev_atten.z()));
     if(!entering && current_layer != -1) {
       r_in.pri_stack->erase(r_in.pri_stack->begin() + static_cast<size_t>(current_layer));
     }
     return(true);
   }
   
-  Float reflect_prob;
-  if(!entering) {
-    outward_normal = -normal;
-    ni_over_nt = ref_idx / current_ref_idx ;
-  } else {
-    outward_normal = normal;
-    ni_over_nt = current_ref_idx / ref_idx;
-  }
-  vec3 unit_direction = unit_vector(r_in.direction());
-  Float cos_theta = ffmin(dot(-unit_direction, outward_normal), (Float)1.0);
-  Float sin_theta = sqrt(1.0 - cos_theta*cos_theta);
-  if(ni_over_nt * sin_theta <= 1.0 ) {
-    reflect_prob = schlick(cos_theta, ref_idx, current_ref_idx);
-    reflect_prob = ni_over_nt == 1 ? 0 : reflect_prob;
-  } else {
-    reflect_prob = 1.0;
-  }
-  if(!entering) {
+  outward_normal = entering ? wh : -wh;
+  ni_over_nt = entering ? current_ref_idx / ref_idx : ref_idx / current_ref_idx ;
+
+  //Never reflect if ni_over_nt == 1
+  // Float reflect_prob = ni_over_nt != 1 ? FrDielectric(dot(wi, wh), ni_over_nt) : 0.0;
+  Float reflect_prob = FrDielectric(dot(wi, outward_normal), ni_over_nt);
+  
+  //Calculate attenuation color
+  if(entering) {
     Float distance = (offset_p-r_in.point_at_parameter(0)).length();
-    srec.attenuation = vec3(std::exp(-distance * attenuation.x()),
-                            std::exp(-distance * attenuation.y()),
-                            std::exp(-distance * attenuation.z()));
+    srec.attenuation = point3f(std::exp(-distance * attenuation.x()),
+                               std::exp(-distance * attenuation.y()),
+                               std::exp(-distance * attenuation.z()));
   } else {
     if(prev_active != -1) {
       Float distance = (offset_p-r_in.point_at_parameter(0)).length();
-      vec3 prev_atten = r_in.pri_stack->at(prev_active)->attenuation;
+      point3f prev_atten = r_in.pri_stack->at(prev_active)->attenuation;
       
-      srec.attenuation = albedo * vec3(std::exp(-distance * prev_atten.x()),
-                                       std::exp(-distance * prev_atten.y()),
-                                       std::exp(-distance * prev_atten.z()));
+      srec.attenuation = albedo * point3f(std::exp(-distance * prev_atten.x()),
+                                          std::exp(-distance * prev_atten.y()),
+                                          std::exp(-distance * prev_atten.z()));
     } else {
       srec.attenuation = albedo;
     }
@@ -172,12 +189,14 @@ bool dielectric::scatter(const ray& r_in, const hit_record& hrec, scatter_record
     if(entering) {
       r_in.pri_stack->pop_back();
     }
+    vec3f reflected = Reflect(wi, outward_normal);
     srec.specular_ray = ray(offset_p, reflected, r_in.pri_stack, r_in.time());
   } else {
     if(!entering && current_layer != -1) {
       r_in.pri_stack->erase(r_in.pri_stack->begin() + current_layer);
     }
-    vec3 refracted = refract(unit_direction, outward_normal, ni_over_nt);
+    vec3f refracted;
+    Refract(wi, outward_normal, ni_over_nt, &refracted);
     srec.specular_ray = ray(offset_p, refracted, r_in.pri_stack, r_in.time());
   }
   return(true);
@@ -186,10 +205,12 @@ bool dielectric::scatter(const ray& r_in, const hit_record& hrec, scatter_record
 
 bool dielectric::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, Sampler* sampler) {
   srec.is_specular = true;
-  vec3 outward_normal;
-  vec3 normal = !hrec.has_bump ? hrec.normal : hrec.bump_normal;
+  normal3f outward_normal;
+  vec3f wi = -unit_vector(r_in.direction());
+  
+  normal3f wh = !hrec.has_bump ? hrec.normal : hrec.bump_normal;
+  wh.make_unit_vector();
   
-  vec3 reflected = reflect(r_in.direction(), normal);
   Float ni_over_nt;
   srec.attenuation = albedo;
   Float current_ref_idx = 1.0;
@@ -200,8 +221,11 @@ bool dielectric::scatter(const ray& r_in, const hit_record& hrec, scatter_record
   int prev_active = -1;  //keeping track of index of active material (higher priority)
   
   bool entering = dot(hrec.normal, r_in.direction()) < 0;
+
   bool skip = false;
-  vec3 offset_p = hrec.p;
+  point3f offset_p = hrec.p;
+  // point3f offset_p = OffsetRayOrigin(hrec.p, hrec.pError, hrec.normal, reflected);
+  
   
   for(size_t i = 0; i < r_in.pri_stack->size(); i++) {
     if(r_in.pri_stack->at(i) == this) {
@@ -224,8 +248,8 @@ bool dielectric::scatter(const ray& r_in, const hit_record& hrec, scatter_record
   if(skip) {
     srec.specular_ray = ray(offset_p, r_in.direction(), r_in.pri_stack, r_in.time());
     Float distance = (offset_p-r_in.point_at_parameter(0)).length();
-    vec3 prev_atten = r_in.pri_stack->at(prev_active)->attenuation;
-    srec.attenuation = vec3(std::exp(-distance * prev_atten.x()),
+    point3f prev_atten = r_in.pri_stack->at(prev_active)->attenuation;
+    srec.attenuation = point3f(std::exp(-distance * prev_atten.x()),
                             std::exp(-distance * prev_atten.y()),
                             std::exp(-distance * prev_atten.z()));
     if(!entering && current_layer != -1) {
@@ -233,51 +257,42 @@ bool dielectric::scatter(const ray& r_in, const hit_record& hrec, scatter_record
     }
     return(true);
   }
+  outward_normal = entering ? wh : -wh;
+  ni_over_nt = entering ? current_ref_idx / ref_idx : ref_idx / current_ref_idx ;
+  //Never reflect if ni_over_nt == 1
+  // Float reflect_prob = ni_over_nt != 1 ? FrDielectric(dot(wi, wh), ni_over_nt) : 0.0;
+  Float reflect_prob = FrDielectric(dot(wi, outward_normal), ni_over_nt);
   
-  Float reflect_prob;
-  if(!entering) {
-    outward_normal = -normal;
-    ni_over_nt = ref_idx / current_ref_idx ;
-  } else {
-    outward_normal = normal;
-    ni_over_nt = current_ref_idx / ref_idx;
-  }
-  vec3 unit_direction = unit_vector(r_in.direction());
-  Float cos_theta = ffmin(dot(-unit_direction, outward_normal), (Float)1.0);
-  Float sin_theta = sqrt(1.0 - cos_theta*cos_theta);
-  if(ni_over_nt * sin_theta <= 1.0 ) {
-    reflect_prob = schlick(cos_theta, ref_idx, current_ref_idx);
-    reflect_prob = ni_over_nt == 1 ? 0 : reflect_prob;
-  } else {
-    reflect_prob = 1.0;
-  }
+  //Calculate attenuation color
   if(!entering) {
     Float distance = (offset_p-r_in.point_at_parameter(0)).length();
-    srec.attenuation = vec3(std::exp(-distance * attenuation.x()),
-                            std::exp(-distance * attenuation.y()),
-                            std::exp(-distance * attenuation.z()));
+    srec.attenuation = point3f(std::exp(-distance * attenuation.x()),
+                               std::exp(-distance * attenuation.y()),
+                               std::exp(-distance * attenuation.z()));
   } else {
     if(prev_active != -1) {
       Float distance = (offset_p-r_in.point_at_parameter(0)).length();
-      vec3 prev_atten = r_in.pri_stack->at(prev_active)->attenuation;
+      point3f prev_atten = r_in.pri_stack->at(prev_active)->attenuation;
       
-      srec.attenuation = albedo * vec3(std::exp(-distance * prev_atten.x()),
-                                       std::exp(-distance * prev_atten.y()),
-                                       std::exp(-distance * prev_atten.z()));
+      srec.attenuation = albedo * point3f(std::exp(-distance * prev_atten.x()),
+                                          std::exp(-distance * prev_atten.y()),
+                                          std::exp(-distance * prev_atten.z()));
     } else {
       srec.attenuation = albedo;
     }
   }
-  if(rng.unif_rand() < reflect_prob) {
+  if(sampler->Get1D() < reflect_prob) {
     if(entering) {
       r_in.pri_stack->pop_back();
     }
+    vec3f reflected = Reflect(wi, outward_normal);
     srec.specular_ray = ray(offset_p, reflected, r_in.pri_stack, r_in.time());
   } else {
     if(!entering && current_layer != -1) {
       r_in.pri_stack->erase(r_in.pri_stack->begin() + current_layer);
     }
-    vec3 refracted = refract(unit_direction, outward_normal, ni_over_nt);
+    vec3f refracted;    
+    Refract(wi, outward_normal, ni_over_nt, &refracted);
     srec.specular_ray = ray(offset_p, refracted, r_in.pri_stack, r_in.time());
   }
   return(true);
@@ -287,16 +302,16 @@ bool dielectric::scatter(const ray& r_in, const hit_record& hrec, scatter_record
 //Diffuse Light
 //
 
-vec3 diffuse_light::emitted(const ray& r_in, const hit_record& rec, Float u, Float v, const vec3& p, bool& is_invisible) {
+point3f diffuse_light::emitted(const ray& r_in, const hit_record& rec, Float u, Float v, const point3f& p, bool& is_invisible) {
   is_invisible = invisible;
   if(dot(rec.normal, r_in.direction()) < 0.0) {
     return(emit->value(u,v,p) * intensity);
   } else {
-    return(vec3(0,0,0));
+    return(point3f(0,0,0));
   }
 }
 
-vec3 diffuse_light::get_albedo(const ray& r_in, const hit_record& rec) const {
+point3f diffuse_light::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(emit->value(rec.u, rec.v, rec.p));
 }
 
@@ -304,16 +319,16 @@ vec3 diffuse_light::get_albedo(const ray& r_in, const hit_record& rec) const {
 //Spot Light
 //
 
-vec3 spot_light::emitted(const ray& r_in, const hit_record& rec, Float u, Float v, const vec3& p, bool& is_invisible) {
+point3f spot_light::emitted(const ray& r_in, const hit_record& rec, Float u, Float v, const point3f& p, bool& is_invisible) {
   is_invisible = invisible;
   if(dot(rec.normal, r_in.direction()) < 0.0) {
     return(falloff(r_in.origin() - rec.p) * emit->value(u,v,p) );
   } else {
-    return(vec3(0,0,0));
+    return(vec3f(0,0,0));
   }
 }
 
-Float spot_light::falloff(const vec3 &w) const {
+Float spot_light::falloff(const vec3f &w) const {
   Float cosTheta = dot(spot_direction, unit_vector(w));
   if (cosTheta < cosTotalWidth) {
     return(0);
@@ -325,7 +340,7 @@ Float spot_light::falloff(const vec3 &w) const {
   return((delta * delta) * (delta * delta));
 }
 
-vec3 spot_light::get_albedo(const ray& r_in, const hit_record& rec) const {
+point3f spot_light::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(emit->value(rec.u, rec.v, rec.p));
 }
 
@@ -345,10 +360,11 @@ bool isotropic::scatter(const ray& r_in, const hit_record& rec, scatter_record&
   srec.attenuation = albedo->value(rec.u,rec.v,rec.p);
   return(true);
 }
-vec3 isotropic::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
+
+point3f isotropic::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
   return(albedo->value(rec.u,rec.v,rec.p) * 0.25 * M_1_PI);
 }
-vec3 isotropic::get_albedo(const ray& r_in, const hit_record& rec) const {
+point3f isotropic::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
@@ -370,15 +386,15 @@ bool orennayar::scatter(const ray& r_in, const hit_record& hrec, scatter_record&
   return(true);
 }
 
-vec3 orennayar::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
+point3f orennayar::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
   onb uvw;
   if(!rec.has_bump) {
     uvw.build_from_w(rec.normal);
   } else {
     uvw.build_from_w(rec.bump_normal);
   }
-  vec3 wi = -unit_vector(uvw.world_to_local(r_in.direction()));
-  vec3 wo = unit_vector(uvw.world_to_local(scattered.direction()));
+  vec3f wi = -unit_vector(uvw.world_to_local(r_in.direction()));
+  vec3f wo = unit_vector(uvw.world_to_local(scattered.direction()));
   
   Float cosine = wo.z();
   
@@ -408,7 +424,7 @@ vec3 orennayar::f(const ray& r_in, const hit_record& rec, const ray& scattered)
   return(albedo->value(rec.u, rec.v, rec.p) * (A + B * maxCos * sinAlpha * tanBeta ) * cosine * M_1_PI );
 }
 
-vec3 orennayar::get_albedo(const ray& r_in, const hit_record& rec) const {
+point3f orennayar::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
@@ -421,9 +437,9 @@ bool MicrofacetReflection::scatter(const ray& r_in, const hit_record& hrec, scat
   srec.is_specular = false;
   srec.attenuation = albedo->value(hrec.u, hrec.v, hrec.p);
   if(!hrec.has_bump) {
-    srec.pdf_ptr = new micro_pdf(hrec.normal, r_in.direction(), distribution);
+    srec.pdf_ptr = new micro_pdf(hrec.normal, r_in.direction(), distribution, hrec.u, hrec.v);
   } else {
-    srec.pdf_ptr = new micro_pdf(hrec.bump_normal, r_in.direction(), distribution);
+    srec.pdf_ptr = new micro_pdf(hrec.bump_normal, r_in.direction(), distribution, hrec.u, hrec.v);
   }
   return(true);
 }
@@ -432,42 +448,128 @@ bool MicrofacetReflection::scatter(const ray& r_in, const hit_record& hrec, scat
   srec.is_specular = false;
   srec.attenuation = albedo->value(hrec.u, hrec.v, hrec.p);
   if(!hrec.has_bump) {
-    srec.pdf_ptr = new micro_pdf(hrec.normal, r_in.direction(), distribution);
+    srec.pdf_ptr = new micro_pdf(hrec.normal, r_in.direction(), distribution, hrec.u, hrec.v);
   } else {
-    srec.pdf_ptr = new micro_pdf(hrec.bump_normal, r_in.direction(), distribution);
+    srec.pdf_ptr = new micro_pdf(hrec.bump_normal, r_in.direction(), distribution, hrec.u, hrec.v);
   }
   return(true);
 }
 
-vec3 MicrofacetReflection::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
+point3f MicrofacetReflection::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
   onb uvw;
   if(!rec.has_bump) {
     uvw.build_from_w(rec.normal);
   } else {
     uvw.build_from_w(rec.bump_normal);
   }
-  vec3 wi = -unit_vector(uvw.world_to_local(r_in.direction()));
-  vec3 wo = unit_vector(uvw.world_to_local(scattered.direction()));
+  vec3f wi = -unit_vector(uvw.world_to_local(r_in.direction()));
+  vec3f wo = unit_vector(uvw.world_to_local(scattered.direction()));
   
   Float cosThetaO = AbsCosTheta(wo);
   Float cosThetaI = AbsCosTheta(wi);
-  vec3 normal = unit_vector(wi + wo);
+  vec3f normal = unit_vector(wi + wo);
   if (cosThetaI == 0 || cosThetaO == 0) {
-    return(vec3(0,0,0));
+    return(vec3f(0,0,0));
   }
   if (normal.x() == 0 && normal.y() == 0 && normal.z() == 0) {
-    return(vec3(0,0,0));
+    return(vec3f(0,0,0));
   }
-  vec3 F = FrCond(cosThetaO, eta, k);
+  point3f F = FrCond(cosThetaO, eta, k);
   Float G = distribution->G(wo,wi,normal);
-  Float D = distribution->D(normal);
+  Float D = distribution->D(normal, rec.u, rec.v);
   return(albedo->value(rec.u, rec.v, rec.p) * F * G * D  * cosThetaI / (4 * CosTheta(wo) * CosTheta(wi) ));
 }
 
-vec3 MicrofacetReflection::get_albedo(const ray& r_in, const hit_record& rec) const {
+point3f MicrofacetReflection::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
+//
+//MicrofacetTransmission
+//
+
+bool MicrofacetTransmission::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, random_gen& rng) {
+  srec.is_specular = false;
+  srec.attenuation = albedo->value(hrec.u, hrec.v, hrec.p);
+  if(!hrec.has_bump) {
+    srec.pdf_ptr = new micro_transmission_pdf(hrec.normal, r_in.direction(), distribution, eta, hrec.u, hrec.v);
+  } else {
+    srec.pdf_ptr = new micro_transmission_pdf(hrec.bump_normal, r_in.direction(), distribution, eta, hrec.u, hrec.v);
+  }
+  return(true);
+}
+
+bool MicrofacetTransmission::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, Sampler* sampler) {
+  srec.is_specular = false;
+  srec.attenuation = albedo->value(hrec.u, hrec.v, hrec.p);
+
+  if(!hrec.has_bump) {
+    srec.pdf_ptr = new micro_transmission_pdf(hrec.normal, r_in.direction(), distribution, eta, hrec.u, hrec.v);
+  } else {
+    srec.pdf_ptr = new micro_transmission_pdf(hrec.bump_normal, r_in.direction(), distribution, eta, hrec.u, hrec.v);
+  }
+  return(true);
+}
+
+point3f MicrofacetTransmission::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
+  onb uvw;
+  if(!rec.has_bump) {
+    uvw.build_from_w(rec.normal);
+  } else {
+    uvw.build_from_w(rec.bump_normal);
+  }
+  
+  vec3f wi = -unit_vector(uvw.world_to_local(r_in.direction()));
+  vec3f wo = unit_vector(uvw.world_to_local(scattered.direction()));
+  bool entering = CosTheta(wi) > 0;
+  Float cosThetaO = CosTheta(wo);
+  Float cosThetaI = CosTheta(wi);
+  if (cosThetaI == 0 || cosThetaO == 0) return point3f(0);
+  bool reflect = cosThetaI * cosThetaO > 0;
+  
+  // Compute $\wh$ from $\wo$ and $\wi$ for microfacet transmission
+  //From pbrt: etaA is incident (etaI) direction if entering, otherwise etaB is
+  // Float eta = CosTheta(wo) > 0 ? (etaB / etaA) : (etaA / etaB); 
+  Float eta2 = 1;
+  if (!reflect) {
+    eta2 = entering ? (1.0/eta) : (eta);
+  }
+  vec3f wh = unit_vector(wi * eta2 + wo);
+  wh = Faceforward(wh, normal3f(0, 0, 1));
+  Float F = FrDielectric(-dot(wo,wh), eta);
+  Float G = distribution->G(wo,wi,wh);
+  Float D = distribution->D(wh, rec.u, rec.v);
+  
+  Float distance = (rec.p-r_in.point_at_parameter(0)).length();
+  
+  point3f atten = !entering ? point3f(std::exp(-distance * k.x()),
+                                      std::exp(-distance * k.y()),
+                                      std::exp(-distance * k.z())) :
+    point3f(1.0);
+  
+  // Same side?
+  if (reflect) {
+    return(F * G * D / (4  * AbsDot(wo,wh)) );
+  }
+
+  Float sqrtDenom = dot(wi, wh)  + dot(wo, wh)* eta2 ;
+  return ((1.0 - F) *
+          albedo->value(rec.u, rec.v, rec.p) * atten *
+          D * G * AbsCosTheta(wo) *
+        std::fabs(eta2 * eta2 *
+       dot(wi, wh) * dot(wo, wh)) /
+      (cosThetaI * cosThetaO * sqrtDenom * sqrtDenom));
+}
+
+point3f MicrofacetTransmission::get_albedo(const ray& r_in, const hit_record& rec) const {
+  return(albedo->value(rec.u, rec.v, rec.p));
+}
+
+point3f MicrofacetTransmission::SchlickFresnel(Float cosTheta) const {
+  auto pow5 = [](Float v) { return (v * v) * (v * v) * v; };
+  return pow5(1 - cosTheta) * (point3f(1.0f));
+}
+
 //
 //Glossy
 //
@@ -476,53 +578,53 @@ vec3 MicrofacetReflection::get_albedo(const ray& r_in, const hit_record& rec) co
 bool glossy::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, random_gen& rng) {
   srec.is_specular = false;
   srec.attenuation = albedo->value(hrec.u, hrec.v, hrec.p);
-  srec.pdf_ptr = new glossy_pdf(hrec.normal, r_in.direction(), distribution);
+  srec.pdf_ptr = new glossy_pdf(hrec.normal, r_in.direction(), distribution, hrec.u, hrec.v);
   return(true);
 }
 
 bool glossy::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, Sampler* sampler) {
   srec.is_specular = false;
   srec.attenuation = albedo->value(hrec.u, hrec.v, hrec.p);
-  srec.pdf_ptr = new glossy_pdf(hrec.normal, r_in.direction(), distribution);
+  srec.pdf_ptr = new glossy_pdf(hrec.normal, r_in.direction(), distribution, hrec.u, hrec.v);
   return(true);
 }
 
-vec3 glossy::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
+point3f glossy::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
   onb uvw;
   if(!rec.has_bump) {
     uvw.build_from_w(rec.normal);
   } else {
     uvw.build_from_w(rec.bump_normal);
   }
-  vec3 wi = -unit_vector(uvw.world_to_local(r_in.direction()));
-  vec3 wo = unit_vector(uvw.world_to_local(scattered.direction()));
+  vec3f wi = -unit_vector(uvw.world_to_local(r_in.direction()));
+  vec3f wo = unit_vector(uvw.world_to_local(scattered.direction()));
   
   auto pow5 = [](Float v) { return (v * v) * (v * v) * v; };
-  vec3 diffuse = (28.0f/(23.0f*M_PI)) * albedo->value(rec.u, rec.v, rec.p) *
-    (vec3(1.0f) - Rs) *
+  point3f diffuse = (28.0f/(23.0f*M_PI)) * albedo->value(rec.u, rec.v, rec.p) *
+    (point3f(1.0f) + -Rs) *
     (1.0 - pow5(1 - 0.5f * AbsCosTheta(wi))) *
     (1.0 - pow5(1 - 0.5f * AbsCosTheta(wo)));
-  vec3 wh = unit_vector(wi + wo);
+  vec3f wh = unit_vector(wi + wo);
   if (wh.x() == 0 && wh.y() == 0 && wh.z() == 0) {
-    return(vec3(0.0f));
+    return(vec3f(0.0f));
   }
   Float cosine  = dot(wh,wi);
   if(cosine < 0) {
     cosine = 0;
   }
-  vec3 specular = distribution->D(wh) /
+  point3f specular = distribution->D(wh, rec.u, rec.v) /
     (4 * AbsDot(wi, wh) *
       std::fmax(AbsCosTheta(wi), AbsCosTheta(wo))) *
       SchlickFresnel(dot(wo, wh));
   return((diffuse + specular) * cosine);
 }
 
-vec3 glossy::SchlickFresnel(Float cosTheta) const {
+point3f glossy::SchlickFresnel(Float cosTheta) const {
   auto pow5 = [](Float v) { return (v * v) * (v * v) * v; };
-  return Rs + pow5(1 - cosTheta) * (vec3(1.0f) - Rs);
+  return Rs + pow5(1 - cosTheta) * (point3f(1.0f) + -Rs);
 }
 
-vec3 glossy::get_albedo(const ray& r_in, const hit_record& rec) const {
+point3f glossy::get_albedo(const ray& r_in, const hit_record& rec) const {
   return(albedo->value(rec.u, rec.v, rec.p));
 }
 
@@ -540,13 +642,13 @@ std::array<Float, pMax + 1> hair::ComputeApPdf(Float cosThetaO, Float h) const {
   Float cosGammaT = SafeSqrt(1 - Sqr(sinGammaT));
   
   // Compute the transmittance _T_ of a single path through the cylinder
-  vec3 T = Exp(-sigma_a * (2 * cosGammaT / cosThetaT));
-  std::array<vec3, pMax + 1> ap = Ap(cosThetaO, eta, h, T);
+  point3f T = Exp(-sigma_a * (2 * cosGammaT / cosThetaT));
+  std::array<point3f, pMax + 1> ap = Ap(cosThetaO, eta, h, T);
   
   // Compute $A_p$ PDF from individual $A_p$ terms
   std::array<Float, pMax + 1> apPdf;
   Float sumY = std::accumulate(ap.begin(), ap.end(), Float(0),
-                               [](Float s, const vec3 &ap) { return s + ap.y(); });
+                               [](Float s, const point3f &ap) { return s + ap.y(); });
   for (int i = 0; i <= pMax; ++i) {
     apPdf[i] = ap[i].y() / sumY;
   }
@@ -555,7 +657,7 @@ std::array<Float, pMax + 1> hair::ComputeApPdf(Float cosThetaO, Float h) const {
 
 bool hair::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, random_gen& rng) {
   onb uvw(hrec.dpdu, hrec.dpdv, hrec.normal);
-  vec3 wo = unit_vector(uvw.world_to_local(r_in.direction()));
+  vec3f wo = unit_vector(uvw.world_to_local(r_in.direction()));
 
   Float sinThetaO = wo.x();
   Float cosThetaO = SafeSqrt(1 - Sqr(sinThetaO));
@@ -564,8 +666,8 @@ bool hair::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec
   Float gammaO = SafeASin(h);
   
   // Derive four random samples from _u2_
-  vec2 u2 = vec2(rng.unif_rand(),rng.unif_rand());
-  vec2 u[2] = {DemuxFloat(u2.e[0]), DemuxFloat(u2.e[1])};
+  vec2f u2 = vec2f(rng.unif_rand(),rng.unif_rand());
+  vec2f u[2] = {DemuxFloat(u2.e[0]), DemuxFloat(u2.e[1])};
   
   // Determine which term $p$ to sample for hair scattering
   std::array<Float, pMax + 1> apPdf = ComputeApPdf(cosThetaO, h);
@@ -592,7 +694,7 @@ bool hair::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec
   }
   
   // Sample $M_p$ to compute $\thetai$
-  u[1].e[0] = std::max(u[1].e[0], Float(1e-5));
+  u[1].e[0] = std::fmax(u[1].e[0], Float(1e-5));
   Float cosTheta = 1 + v[p] * std::log(u[1].e[0] + (1 - u[1].e[0]) * std::exp(-2 / v[p]));
   Float sinTheta = SafeSqrt(1 - Sqr(cosTheta));
   Float cosPhi = std::cos(2 * M_PI * u[1].e[1]);
@@ -614,10 +716,10 @@ bool hair::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec
   
   // Compute _wi_ from sampled hair scattering angles
   Float phiI = phiO + dphi;
-  vec3 wi(sinThetaI, cosThetaI * std::cos(phiI),
+  vec3f wi(sinThetaI, cosThetaI * std::cos(phiI),
           cosThetaI * std::sin(phiI));
   srec.is_specular = false;
-  srec.attenuation = vec3(1,1,1);
+  srec.attenuation = vec3f(1,1,1);
   
   srec.pdf_ptr = new hair_pdf(uvw, wi, wo, 
                               eta, h, gammaO,  s, sigma_a,
@@ -628,7 +730,7 @@ bool hair::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec
 
 bool hair::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, Sampler* sampler) {
   onb uvw(hrec.dpdu, hrec.dpdv, hrec.normal);
-  vec3 wo = unit_vector(uvw.world_to_local(r_in.direction()));
+  vec3f wo = unit_vector(uvw.world_to_local(r_in.direction()));
   
   Float sinThetaO = wo.x();
   Float cosThetaO = SafeSqrt(1 - Sqr(sinThetaO));
@@ -637,8 +739,8 @@ bool hair::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec
   Float gammaO = SafeASin(h);
   
   // Derive four random samples from _u2_
-  vec2 u2 = vec2(sampler->Get1D(),sampler->Get1D());
-  vec2 u[2] = {DemuxFloat(u2.e[0]), DemuxFloat(u2.e[1])};
+  vec2f u2 = vec2f(sampler->Get1D(),sampler->Get1D());
+  vec2f u[2] = {DemuxFloat(u2.e[0]), DemuxFloat(u2.e[1])};
   
   // Determine which term $p$ to sample for hair scattering
   std::array<Float, pMax + 1> apPdf = ComputeApPdf(cosThetaO, h);
@@ -665,7 +767,7 @@ bool hair::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec
   }
   
   // Sample $M_p$ to compute $\thetai$
-  u[1].e[0] = std::max(u[1].e[0], Float(1e-5));
+  u[1].e[0] = std::fmax(u[1].e[0], Float(1e-5));
   Float cosTheta = 1 + v[p] * std::log(u[1].e[0] + (1 - u[1].e[0]) * std::exp(-2 / v[p]));
   Float sinTheta = SafeSqrt(1 - Sqr(cosTheta));
   Float cosPhi = std::cos(2 * M_PI * u[1].e[1]);
@@ -687,10 +789,10 @@ bool hair::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec
   
   // Compute _wi_ from sampled hair scattering angles
   Float phiI = phiO + dphi;
-  vec3 wi(sinThetaI, cosThetaI * std::cos(phiI),
+  vec3f wi(sinThetaI, cosThetaI * std::cos(phiI),
           cosThetaI * std::sin(phiI));
   srec.is_specular = false;
-  srec.attenuation = vec3(1,1,1);
+  srec.attenuation = vec3f(1,1,1);
   
   srec.pdf_ptr = new hair_pdf(uvw, wi, wo, 
                               eta, h, gammaO,  s, sigma_a,
@@ -699,12 +801,12 @@ bool hair::scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec
 }
 
 
-vec3 hair::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
+point3f hair::f(const ray& r_in, const hit_record& rec, const ray& scattered) const {
   onb uvw(rec.dpdu, rec.dpdv, rec.normal);
-  // vec3 wo = -unit_vector(uvw.world_to_local(r_in.direction()));
-  vec3 wo = -unit_vector(uvw.world_to_local(r_in.direction()));
+  // vec3f wo = -unit_vector(uvw.world_to_local(r_in.direction()));
+  vec3f wo = -unit_vector(uvw.world_to_local(r_in.direction()));
   
-  vec3 wi = unit_vector(uvw.world_to_local(scattered.direction()));
+  vec3f wi = unit_vector(uvw.world_to_local(scattered.direction()));
   
   Float h = -1 + 2 * rec.v;
   Float gammaO = SafeASin(h);
@@ -729,12 +831,12 @@ vec3 hair::f(const ray& r_in, const hit_record& rec, const ray& scattered) const
   Float gammaT = SafeASin(sinGammaT);
   
   // Compute the transmittance _T_ of a single path through the cylinder
-  vec3 T = Exp(-sigma_a * (2 * cosGammaT / cosThetaT));
+  point3f T = Exp(-sigma_a * (2 * cosGammaT / cosThetaT));
   
   // Evaluate hair BSDF
   Float phi = phiI - phiO;
-  std::array<vec3, pMax + 1> ap = Ap(cosThetaO, eta, h, T);
-  vec3 fsum(0.0);
+  std::array<point3f, pMax + 1> ap = Ap(cosThetaO, eta, h, T);
+  point3f fsum(0.0);
   for (int p = 0; p < pMax; ++p) {
     // Compute $\sin \thetao$ and $\cos \thetao$ terms accounting for scales
     Float sinThetaOp, cosThetaOp;
@@ -756,7 +858,7 @@ vec3 hair::f(const ray& r_in, const hit_record& rec, const ray& scattered) const
     }
     
     // Handle out-of-range $\cos \thetao$ from scale adjustment
-    cosThetaOp = std::abs(cosThetaOp);
+    cosThetaOp = std::fabs(cosThetaOp);
     fsum += Mp(cosThetaI, cosThetaOp, sinThetaI, sinThetaOp, v[p]) * ap[p] *
       Np(phi, p, s, gammaO, gammaT);
   }
@@ -769,8 +871,8 @@ vec3 hair::f(const ray& r_in, const hit_record& rec, const ray& scattered) const
   return(fsum);
 }
 
-// vec3 hair::SigmaAFromConcentration(Float ce, Float cp) {
-//   vec3 sigma_a;
+// vec3f hair::SigmaAFromConcentration(Float ce, Float cp) {
+//   vec3f sigma_a;
 //   Float eumelaninSigmaA[3] = {0.419f, 0.697f, 1.37f};
 //   Float pheomelaninSigmaA[3] = {0.187f, 0.4f, 1.05f};
 //   for (int i = 0; i < 3; ++i)
@@ -778,8 +880,8 @@ vec3 hair::f(const ray& r_in, const hit_record& rec, const ray& scattered) const
 //   return(sigma_a);
 // }
 // 
-// vec3 hair::SigmaAFromReflectance(const vec3 &c, Float beta_n) {
-//   vec3 sigma_a;
+// vec3f hair::SigmaAFromReflectance(const vec3f &c, Float beta_n) {
+//   vec3f sigma_a;
 //   for (int i = 0; i < 3; ++i)
 //     sigma_a.e[i] = Sqr(std::log(c.e[i]) /
 //       (5.969f - 0.215f * beta_n + 2.532f * Sqr(beta_n) -
