@@ -6,11 +6,17 @@
 #include <cstring>
 #include "vec3.h"
 #include "vec2.h"
+#include "point3.h"
+#include "point2.h"
+
+#include "normal.h"
+#include <limits>
 #include <array>
 
 static const Float mpi_over_180 = M_PI/180;
 static const Float SqrtPiOver8 = 0.626657069f;
 static const Float ONE_OVER_2_PI = 1 / (2 * M_PI);
+static const Float Infinity =  Infinity;
 
 #ifndef FLOATDEF
 #define FLOATDEF
@@ -21,6 +27,12 @@ typedef float Float;
 #endif 
 #endif
 
+static constexpr Float MachineEpsilon = std::numeric_limits<Float>::epsilon() * 0.5;
+
+inline constexpr Float gamma(int n) {
+  return (n * MachineEpsilon) / (1 - n * MachineEpsilon);
+}
+
 template<class T>
 inline T ffmin(T a, T b) { return(a < b ? a : b);}
 
@@ -36,22 +48,30 @@ template <typename T> int sgn(T val) {
   return (T(0) < val) - (val < T(0));
 }
 
-inline vec3 sgn(vec3 v) {
-  return(vec3(sgn(v.x()),sgn(v.y()),sgn(v.z())));
+inline vec3f sgn(vec3f v) {
+  return(vec3f(sgn(v.x()),sgn(v.y()),sgn(v.z())));
+}
+
+inline Float Radians(Float deg) { 
+  return (M_PI / 180) * deg; 
+}
+inline Float Degrees(Float rad) { 
+  return (180 / M_PI) * rad; 
 }
 
 
-inline vec3 de_nan(const vec3& c) {
-  vec3 temp = c;
-  if(std::isnan(c[0])) temp.e[0] = 0.0f;
+
+inline point3f de_nan(const point3f& c) {
+  point3f temp = c;
+  if(std::isnan(c[0])) temp.e[0] = 1.0f;
   if(std::isnan(c[1])) temp.e[1] = 0.0f;
-  if(std::isnan(c[2])) temp.e[2] = 0.0f;
+  if(std::isnan(c[2])) temp.e[2] = 1.0f;
   return(temp);
 }
 
 
 
-inline vec3 rand_to_unit(vec2 u) {
+inline vec3f rand_to_unit(vec2f u) {
   Float a = 2.0*u.x() - 1.0; 
   Float b = 2.0*u.y() - 1.0;
   if (b == 0) {
@@ -65,29 +85,61 @@ inline vec3 rand_to_unit(vec2 u) {
     r = b;
     phi = (M_PI_2) - (M_PI_4)*(a/b);
   }
-  return(vec3(r*cos(phi),r*sin(phi),0));
+  return(vec3f(r*cos(phi),r*sin(phi),0));
 }
 
-inline vec3 rand_cosine_direction(vec2 u) {
+inline vec3f rand_cosine_direction(vec2f u) {
   Float r1 = u.x();
   Float r2 = u.y();
   Float z = std::sqrt(1.0-r2);
   Float phi = 2.0 * M_PI * r1;
   Float x = cos(phi) * std::sqrt(r2);
   Float y = sin(phi) * std::sqrt(r2);
-  return(vec3(x, y, z));
+  return(vec3f(x, y, z));
 }
 
-inline vec3 rand_to_sphere(Float radius, Float distance_squared, vec2 u) {
+inline vec3f rand_to_sphere(Float radius, Float distance_squared, vec2f u) {
   Float r1 = u.x();
   Float r2 = u.y();
   Float z = 1.0 + r2 * (std::sqrt(1.0-radius * radius / distance_squared) - 1);
   Float phi = 2.0 * M_PI * r1;
   Float x = std::cos(phi) * std::sqrt(1-z*z);
   Float y = std::sin(phi) * std::sqrt(1-z*z);
-  return(vec3(x,y,z));
+  return(vec3f(x,y,z));
 }
 
+inline vec3f SphericalDirection(Float sinTheta, 
+                                   Float cosTheta, Float phi) {
+  return vec3f(sinTheta * std::cos(phi), 
+               sinTheta * std::sin(phi),
+               cosTheta);
+}
+
+inline vec3f SphericalDirection(Float sinTheta, Float cosTheta, 
+                                Float phi, const vec3f &x, const vec3f &y,
+                                const vec3f &z) {
+  return sinTheta * std::cos(phi) * x + sinTheta * std::sin(phi) * y + cosTheta * z;
+}
+
+// 
+// point2f ConcentricSampleDisk(const point2f &u) {
+//   point2f uOffset = 2.f * u - vec2f(1, 1);
+//   
+//   if (uOffset.x() == 0 && uOffset.y() == 0) {
+//     return(point2f(0, 0));
+//   }
+//   
+//   Float theta, r;
+//   if (std::fabs(uOffset.x()) > std::fabs(uOffset.y())) {
+//     r = uOffset.x();
+//     theta = M_PI_4 * (uOffset.y() / uOffset.x());
+//   } else {
+//     r = uOffset.y();
+//     theta = M_PI_2 - M_PI_4 * (uOffset.x() / uOffset.y());
+//   }
+//   return(r * point2f(std::cos(theta), std::sin(theta)));
+//   
+// }
 
 
 // template<class T>
@@ -101,15 +153,37 @@ inline Float saturate(Float v1) {
   return(v1);
 }
 
-inline vec3 saturate(vec3 v1) {
+inline vec3f saturate(vec3f v1) {
   v1.e[0] = saturate(v1.e[0]);
   v1.e[1] = saturate(v1.e[1]);
   v1.e[2] = saturate(v1.e[2]);
   return(v1);
 }
 
-inline vec3 clamp(const vec3& c, Float clamplow, Float clamphigh) {
-  vec3 temp = c;
+inline vec3f clamp(const vec3f& c, Float clamplow, Float clamphigh) {
+  vec3f temp = c;
+  if(c.e[0] > clamphigh) {
+    temp.e[0] = clamphigh;
+  } else if(c[0] < clamplow) {
+    temp.e[0] = clamplow;
+  }
+  if(c.e[1] > clamphigh) {
+    temp.e[1] = clamphigh;
+  } else if(c[1] < clamplow) {
+    temp.e[1] = clamplow;
+  }
+  if(c.e[2] > clamphigh) {
+    temp.e[2] = clamphigh;
+  } else if(c[2] < clamplow) {
+    temp.e[2] = clamplow;
+  }
+  return(temp);
+}
+
+
+
+inline point3f clamp_point(const point3f& c, Float clamplow, Float clamphigh) {
+  point3f temp = c;
   if(c.e[0] > clamphigh) {
     temp.e[0] = clamphigh;
   } else if(c[0] < clamplow) {
@@ -137,48 +211,106 @@ inline Float clamp(const Float& c, Float clamplow, Float clamphigh) {
   return(c);
 }
 
-inline vec3 clamp(vec3 input, vec3 low, vec3 high) {
-  vec3 final(clamp(input.x(), low.x(), high.x()),
-             clamp(input.y(), low.y(), high.y()),
-             clamp(input.z(), low.z(), high.z()));
+inline vec3f clamp(vec3f input, vec3f low, vec3f high) {
+  vec3f final(clamp(input.x(), low.x(), high.x()),
+              clamp(input.y(), low.y(), high.y()),
+              clamp(input.z(), low.z(), high.z()));
   return(final);
 }
 
-inline Float CosTheta(const vec3 &w) { return w.z(); }
-inline Float Cos2Theta(const vec3 &w) { return w.z() * w.z(); }
-inline Float AbsCosTheta(const vec3 &w) { return std::fabs(w.z()); }
-
-inline Float Sin2Theta(const vec3 &w) {
-  return(std::max((Float)0, (Float)1 - Cos2Theta(w)));
+inline point3f clamp(point3f input, point3f low, point3f high) {
+  point3f final(clamp(input.x(), low.x(), high.x()),
+                clamp(input.y(), low.y(), high.y()),
+                clamp(input.z(), low.z(), high.z()));
+  return(final);
 }
 
-inline Float SinTheta(const vec3 &w) {
+
+
+// General Utility Functions
+inline Float Sqr(Float v) { return v * v; }
+template <int n>
+static Float Pow(Float v) {
+  static_assert(n > 0, "Power can't be negative");
+  Float n2 = Pow<n / 2>(v);
+  return n2 * n2 * Pow<n & 1>(v);
+}
+
+template <>
+inline Float Pow<1>(Float v) {
+  return(v);
+}
+
+template <>
+inline Float Pow<0>(Float v) {
+  return(1);
+}
+
+inline Float SafeASin(Float x) {
+  return(std::asin(clamp(x, -1, 1)));
+}
+
+inline Float SafeSqrt(Float x) {
+  return(std::sqrt(std::fmax(Float(0), x)));
+}
+
+inline vec3f Exp(vec3f a) {
+  return(vec3f(std::exp(a.x()),std::exp(a.y()),std::exp(a.z())));
+}
+
+inline point3f Exp(point3f a) {
+  return(point3f(std::exp(a.x()),std::exp(a.y()),std::exp(a.z())));
+}
+
+inline Float Logistic(Float x, Float s) {
+  x = std::abs(x);
+  return(std::exp(-x / s) / (s * Sqr(1 + std::exp(-x / s))));
+}
+
+inline Float LogisticCDF(Float x, Float s) {
+  return(1 / (1 + std::exp(-x / s)));
+}
+
+inline Float TrimmedLogistic(Float x, Float s, Float a, Float b) {
+  return(Logistic(x, s) / (LogisticCDF(b, s) - LogisticCDF(a, s)));
+}
+
+
+inline Float CosTheta(const vec3f &w) { return w.z(); }
+inline Float Cos2Theta(const vec3f &w) { return w.z() * w.z(); }
+inline Float AbsCosTheta(const vec3f &w) { return std::fabs(w.z()); }
+
+inline Float Sin2Theta(const vec3f &w) {
+  return(std::fmax((Float)0, (Float)1 - Cos2Theta(w)));
+}
+
+inline Float SinTheta(const vec3f &w) {
   return(std::sqrt(Sin2Theta(w)));
 }
 
-inline Float CosPhi(const vec3 &w) {
+inline Float CosPhi(const vec3f &w) {
   Float sinTheta = SinTheta(w);
   return((sinTheta == 0) ? 1 : clamp(w.x() / sinTheta, -1, 1));
 }
 
-inline Float SinPhi(const vec3 &w) {
+inline Float SinPhi(const vec3f &w) {
   Float sinTheta = SinTheta(w);
   return((sinTheta == 0) ? 0 : clamp(w.y() / sinTheta, -1, 1));
 }
 
-inline Float Cos2Phi(const vec3 &w) {
+inline Float Cos2Phi(const vec3f &w) {
   return(CosPhi(w) * CosPhi(w));
 }
 
-inline Float Sin2Phi(const vec3 &w) {
+inline Float Sin2Phi(const vec3f &w) {
   return(SinPhi(w) * SinPhi(w));
 }
 
-inline Float TanTheta(const vec3 &w) {
+inline Float TanTheta(const vec3f &w) {
   return(SinTheta(w) / CosTheta(w));
 }
 
-inline Float Tan2Theta(const vec3 &w) {
+inline Float Tan2Theta(const vec3f &w) {
   return(Sin2Theta(w) / Cos2Theta(w));
 }
 
@@ -199,6 +331,27 @@ inline Float FrDielectric(Float cosThetaI, Float etaI, Float etaT) {
   return((Rparl * Rparl + Rperp * Rperp) / 2);
 }
 
+
+
+inline Float FrDielectric(Float cosThetaI, Float eta) {
+  cosThetaI = clamp(cosThetaI, -1, 1);
+  // Potentially flip interface orientation for Fresnel equations
+  if (cosThetaI < 0) {
+    eta = 1 / eta;
+    cosThetaI = -cosThetaI;
+  }
+  
+  // Compute $\cos\,\theta_\roman{t}$ for Fresnel equations using Snell's law
+  Float sin2Theta_i = 1 - Sqr(cosThetaI);
+  Float sin2Theta_t = sin2Theta_i * Sqr(eta);
+  if (sin2Theta_t >= 1) return 1.0f;
+  Float cosTheta_t = SafeSqrt(1 - sin2Theta_t);
+  
+  Float r_parl = (cosThetaI - eta * cosTheta_t) / (cosThetaI + eta * cosTheta_t);
+  Float r_perp = (eta * cosThetaI - cosTheta_t) / (eta * cosThetaI + cosTheta_t);
+  return (r_parl * r_parl + r_perp * r_perp) / 2;
+}
+
 inline bool quadratic(Float a, Float b, Float c, Float *t0, Float *t1) {
   Float discrim = DifferenceOfProducts(b, b, 4 * a, c);
   if (discrim < 0) {
@@ -229,30 +382,74 @@ template <typename Predicate> int FindInterval(int size, const Predicate &pred)
   return clamp(first - 1, 0, size - 2);
 }
 
-inline Float SphericalPhi(const vec3 &v) {
+inline Float SphericalPhi(const vec3f &v) {
   float p = atan2f(v.x(), v.y());
   return((p < 0.0f) ? p + 2.0f*M_PI : p);
 }
 
-inline Float SphericalTheta(const vec3 &v) {
+inline Float SphericalTheta(const vec3f &v) {
   return(acosf(clamp(v.z(), -1.0f, 1.0f)));
 }
 
-inline vec3 SphericalDirection(Float sinTheta, Float cosTheta, Float phi) {
-  return(vec3(sinTheta * std::cos(phi),
-              sinTheta * std::sin(phi),
-              cosTheta));
-}
-
-inline bool SameHemisphere(const vec3 &w, const vec3 &wp) {
+inline bool SameHemisphere(const vec3f &w, const vec3f &wp) {
   return(w.z() * wp.z() > 0);
 }
 
+inline bool SameHemisphere(const normal3f &w, const vec3f &wp) {
+  return(w.z() * wp.z() > 0);
+}
 
-inline Float AbsDot(const vec3 &v1, const vec3 &v2) {
+inline bool SameHemisphere(const vec3f &w, const normal3f &wp) {
+  return(w.z() * wp.z() > 0);
+}
+
+inline bool refract(const vec3f& v, const vec3f& n, Float ni_over_nt, vec3f& refracted) {
+  vec3f uv = unit_vector(v);
+  Float dt = dot(uv, n);
+  Float discriminant = 1.0 - ni_over_nt * ni_over_nt * (1 - dt * dt);
+  if(discriminant > 0) {
+    refracted = ni_over_nt * (uv - n * dt) - n * sqrt(discriminant);
+    return(true);
+  } else {
+    return(false);
+  }
+}
+
+inline bool refract(const vec3f& v, const normal3f& n2, Float ni_over_nt, vec3f& refracted) {
+  vec3f uv = unit_vector(v);
+  vec3f n = vec3f(n2.x(),n2.y(),n2.z());
+  Float dt = dot(uv, n);
+  Float discriminant = 1.0 - ni_over_nt * ni_over_nt * (1 - dt * dt);
+  if(discriminant > 0) {
+    refracted = ni_over_nt * (uv - n * dt) - n * sqrt(discriminant);
+    return(true);
+  } else {
+    return(false);
+  }
+}
+
+inline vec3f refract(const vec3f& uv, const vec3f& n, Float ni_over_nt) {
+  Float cos_theta = dot(-uv, n);
+  vec3f r_out_parallel =  ni_over_nt * (uv + cos_theta*n);
+  vec3f r_out_perp = -sqrt(1.0 - r_out_parallel.squared_length()) * n;
+  return(r_out_parallel + r_out_perp);
+}
+
+inline vec3f refract(const vec3f& uv, const normal3f& n, Float ni_over_nt) {
+  vec3f n2 = vec3f(n.x(),n.y(),n.z());
+  Float cos_theta = dot(-uv, n);
+  vec3f r_out_parallel =  ni_over_nt * (uv + cos_theta*n2);
+  vec3f r_out_perp = -sqrt(1.0 - r_out_parallel.squared_length()) * n2;
+  return(r_out_parallel + r_out_perp);
+}
+
+
+template <typename T>
+inline Float AbsDot(const vec3<T> &v1, const vec3<T> &v2) {
   return(std::fabs(dot(v1,v2)));
 } 
 
+
 template <typename IN_T, typename OUT_T>
 inline OUT_T reinterpret_type(const IN_T in) {
   OUT_T out;
@@ -326,7 +523,14 @@ inline Float Erf(Float x) {
   return sign * y;
 }
 
-inline vec3 Reflect(const vec3 &wo, const vec3 &n) {
+template <typename T>
+inline vec3<T> Reflect(const vec3<T> &wo, const normal3<T> &n) {
+  normal3<T> n2 = 2 * dot(wo, n) * n;
+  return(-wo + vec3<T>(n2.x(),n2.y(),n2.z()));
+}
+
+template <typename T>
+inline vec3<T> Reflect(const vec3<T> &wo, const vec3<T> &n) {
   return(-wo + 2 * dot(wo, n) * n);
 }
 
@@ -342,18 +546,46 @@ inline float BitsToFloat(uint32_t ui) {
   return f;
 }
 
+
+inline float NextFloatUp(float v) {
+  // Handle infinity and negative zero for _NextFloatUp()_
+  if (std::isinf(v) && v > 0.) return v;
+  if (v == -0.f) v = 0.f;
+  
+  // Advance _v_ to next higher float
+  uint32_t ui = FloatToBits(v);
+  if (v >= 0) {
+    ++ui;
+  } else {
+    --ui;
+  }
+  return BitsToFloat(ui);
+}
+
+inline float NextFloatDown(float v) {
+  // Handle infinity and positive zero for _NextFloatDown()_
+  if (std::isinf(v) && v < 0.) return v;
+  if (v == 0.f) v = -0.f;
+  uint32_t ui = FloatToBits(v);
+  if (v > 0)
+    --ui;
+  else
+    ++ui;
+  return BitsToFloat(ui);
+}
+
 constexpr Float origin() { return 1.0f / 32.0f; }
 constexpr Float float_scale() { return 1.0f / 65536.0f; }
 constexpr Float int_scale() { return 256.0f; }
 
-inline vec3 offset_ray(const vec3 p, const vec3 n) {
+inline point3f offset_ray(const vec3f p, const normal3f n) {
   int of_i[3] = {(int)(int_scale() * n.x()), (int)(int_scale() * n.y()), (int)(int_scale() * n.z())};
-  vec3 p_i(int_to_float(float_to_int(p.x())+((p.x() < 0) ? -of_i[0] : of_i[0])),
-           int_to_float(float_to_int(p.y())+((p.y() < 0) ? -of_i[1] : of_i[1])),
-           int_to_float(float_to_int(p.z())+((p.z() < 0) ? -of_i[2] : of_i[2])));
-  return(vec3(std::fabs(p.x()) < origin() ? p.x() + float_scale()*n.x() : p_i.x(),
-              std::fabs(p.y()) < origin() ? p.y() + float_scale()*n.y() : p_i.y(),
-              std::fabs(p.z()) < origin() ? p.z() + float_scale()*n.z() : p_i.z()));
+  point3f p_i(int_to_float(float_to_int(p.x())+((p.x() < 0) ? -of_i[0] : of_i[0])),
+              int_to_float(float_to_int(p.y())+((p.y() < 0) ? -of_i[1] : of_i[1])),
+              int_to_float(float_to_int(p.z())+((p.z() < 0) ? -of_i[2] : of_i[2])));
+  return(point3f(std::fabs(p.x()) < origin() ? p.x() + float_scale()*n.x() : p_i.x(),
+                 std::fabs(p.y()) < origin() ? p.y() + float_scale()*n.y() : p_i.y(),
+                 std::fabs(p.z()) < origin() ? p.z() + float_scale()*n.z() : p_i.z()));
 }
 
 inline Float Log2(Float x) {
@@ -361,11 +593,11 @@ inline Float Log2(Float x) {
   return(std::log(x) * invLog2);
 }
 
-inline vec3 RGBtoHSV(vec3& rgb) {
+inline point3f RGBtoHSV(point3f& rgb) {
   Float max_val = ffmax(ffmax(rgb.r(), rgb.g()), rgb.b());
   Float min_val = ffmin(ffmin(rgb.r(), rgb.g()), rgb.b());
   Float delta_val = max_val - min_val;
-  vec3 hsv;
+  point3f hsv;
   
   if(delta_val > 0) {
     if(max_val == rgb.r()) {
@@ -393,85 +625,43 @@ inline vec3 RGBtoHSV(vec3& rgb) {
 }
 
 
-inline vec3 HSVtoRGB(vec3 hsv) {
+inline point3f HSVtoRGB(point3f hsv) {
   Float chroma = hsv.z() * hsv.y(); 
   Float fHPrime = fmod(hsv.x() / 60.0, 6);
   Float x_val = chroma * (1 - std::fabs(fmod(fHPrime, 2) - 1));
   Float m_val = hsv.z() - chroma;
 
   if(0 <= fHPrime && fHPrime < 1) {
-    vec3 rgb(chroma,x_val,0);
+    point3f rgb(chroma,x_val,0);
     rgb += m_val;
     return(rgb);
   } else if(1 <= fHPrime && fHPrime < 2) {
-    vec3 rgb(x_val,chroma,0);
+    point3f rgb(x_val,chroma,0);
     rgb += m_val;
     return(rgb);
   } else if(2 <= fHPrime && fHPrime < 3) {
-    vec3 rgb(0,chroma,x_val);
+    point3f rgb(0,chroma,x_val);
     rgb += m_val;
     return(rgb);
   } else if(3 <= fHPrime && fHPrime < 4) {
-    vec3 rgb(0,x_val,chroma);
+    point3f rgb(0,x_val,chroma);
     rgb += m_val;
     return(rgb);
   } else if(4 <= fHPrime && fHPrime < 5) {
-    vec3 rgb(x_val,0,chroma);
+    point3f rgb(x_val,0,chroma);
     rgb += m_val;
     return(rgb);
   } else if(5 <= fHPrime && fHPrime < 6) {
-    vec3 rgb(chroma,0,x_val);
+    point3f rgb(chroma,0,x_val);
     rgb += m_val;
     return(rgb);
   } else {
-    vec3 rgb(0,0,0);
+    point3f rgb(0,0,0);
     rgb += m_val;
     return(rgb);
   }
 }
 
-// General Utility Functions
-inline Float Sqr(Float v) { return v * v; }
-template <int n>
-static Float Pow(Float v) {
-  static_assert(n > 0, "Power can't be negative");
-  Float n2 = Pow<n / 2>(v);
-  return n2 * n2 * Pow<n & 1>(v);
-}
-
-template <>
-inline Float Pow<1>(Float v) {
-  return(v);
-}
-
-template <>
-inline Float Pow<0>(Float v) {
-  return(1);
-}
-inline Float SafeASin(Float x) {
-  return(std::asin(clamp(x, -1, 1)));
-}
-
-inline Float SafeSqrt(Float x) {
-  return(std::sqrt(std::max(Float(0), x)));
-}
-
-inline vec3 Exp(vec3 a) {
-  return(vec3(std::exp(a.x()),std::exp(a.y()),std::exp(a.z())));
-}
-
-inline Float Logistic(Float x, Float s) {
-  x = std::abs(x);
-  return(std::exp(-x / s) / (s * Sqr(1 + std::exp(-x / s))));
-}
-
-inline Float LogisticCDF(Float x, Float s) {
-  return(1 / (1 + std::exp(-x / s)));
-}
-
-inline Float TrimmedLogistic(Float x, Float s, Float a, Float b) {
-  return(Logistic(x, s) / (LogisticCDF(b, s) - LogisticCDF(a, s)));
-}
 
 //Hair utilities
 static const int pMax = 3;
@@ -493,10 +683,10 @@ inline uint32_t Compact1By1(uint32_t x) {
   return(x);
 }
 
-inline vec2 DemuxFloat(Float f) {
+inline vec2f DemuxFloat(Float f) {
   uint64_t v = f * (1ull << 32);
   uint32_t bits[2] = {Compact1By1(v), Compact1By1(v >> 1)};
-  return(vec2(bits[0] / Float(1 << 16), bits[1] / Float(1 << 16)));
+  return(vec2f(bits[0] / Float(1 << 16), bits[1] / Float(1 << 16)));
 }
 
 inline Float I0(Float x) {
@@ -524,9 +714,9 @@ inline Float LogI0(Float x) {
   }
 }
 
-inline std::array<vec3, pMax + 1> Ap(Float cosThetaO, Float eta, Float h,
-                                     const vec3 &T) {
-  std::array<vec3, pMax + 1> ap;
+inline std::array<point3f, pMax + 1> Ap(Float cosThetaO, Float eta, Float h,
+                                     const point3f &T) {
+  std::array<point3f, pMax + 1> ap;
   // Compute $p=0$ attenuation at initial cylinder intersection
   Float cosGammaO = SafeSqrt(1 - h * h);
   Float cosTheta = cosThetaO * cosGammaO;
@@ -542,7 +732,7 @@ inline std::array<vec3, pMax + 1> Ap(Float cosThetaO, Float eta, Float h,
   }
   
   // Compute attenuation term accounting for remaining orders of scattering
-  ap[pMax] = ap[pMax - 1] * f * T / (vec3(1.0f,1.0f,1.0f) - T * f);
+  ap[pMax] = ap[pMax - 1] * f * T / (point3f(1.0f,1.0f,1.0f) + -T * f);
   return(ap);
 }
 
@@ -606,5 +796,97 @@ inline uint64_t ReverseBits64(uint64_t n) {
   return (n0 << 32) | n1;
 }
 
+template <typename T> inline void
+CoordinateSystem(const vec3<T> &v1, vec3<T> *v2, vec3<T> *v3) {
+  if (std::fabs(v1.x()) > std::fabs(v1.y())) {
+    *v2 = vec3<T>(-v1.z(), 0, v1.x()) /
+      std::sqrt(v1.x() * v1.x() + v1.z() * v1.z());
+  } else {
+    *v2 = vec3<T>(0, v1.z(), -v1.y()) /
+      std::sqrt(v1.y() * v1.y() + v1.z() * v1.z());
+  }
+  *v3 = cross(v1, *v2);
+}
+
+inline point3f OffsetRayOrigin(const point3f &p, const vec3f &pError,
+                               const normal3f &n, const vec3f &w) {
+  Float d = dot(Abs(n), pError);
+  vec3f offset = d * vec3f(n.x(),n.y(),n.z());
+  if (dot(w, n) < 0) {
+    offset = -offset;
+  }
+  point3f po = p + offset;
+  for (int i = 0; i < 3; ++i) {
+    if (offset.e[i] > 0) {
+      po.e[i] = NextFloatUp(po.e[i]);
+    } else if (offset.e[i] < 0)  {
+      po.e[i] = NextFloatDown(po.e[i]);
+    }
+  }
+  
+  return po;
+}
+
+
+inline Float UniformConePdf(Float cosThetaMax) {
+  return 1 / (2 * M_PI * (1 - cosThetaMax));
+}
+
+inline vec3f UniformSampleCone(const point2f &u, Float cosThetaMax) {
+  Float cosTheta = ((Float)1 - u[0]) + u[0] * cosThetaMax;
+  Float sinTheta = std::sqrt((Float)1 - cosTheta * cosTheta);
+  Float phi = u[1] * 2 * M_PI;
+  return vec3f(std::cos(phi) * sinTheta, std::sin(phi) * sinTheta,
+               cosTheta);
+}
+
+inline vec3f UniformSampleCone(const point2f &u, Float cosThetaMax,
+                        const vec3f &x, const vec3f &y,
+                        const vec3f &z) {
+  Float cosTheta = lerp(u[0], cosThetaMax, 1.f);
+  Float sinTheta = std::sqrt((Float)1. - cosTheta * cosTheta);
+  Float phi = u[1] * 2 * M_PI;
+  return std::cos(phi) * sinTheta * x + std::sin(phi) * sinTheta * y +
+    cosTheta * z;
+}
+
+inline vec3f UniformSampleHemisphere(const vec2f &u) {
+  Float z = u[0];
+  Float r = std::sqrt(std::fmax((Float)0, (Float)1. - z * z));
+  Float phi = 2 * M_PI * u[1];
+  return vec3f(r * std::cos(phi), r * std::sin(phi), z);
+}
+
+inline Float UniformHemispherePdf() { return 0.5*M_1_PI; }
+
+inline vec3f UniformSampleSphere(const vec2f &u) {
+  Float z = 1 - 2 * u[0];
+  Float r = std::sqrt(std::fmax((Float)0, (Float)1 - z * z));
+  Float phi = 2 * M_PI * u[1];
+  return vec3f(r * std::cos(phi), r * std::sin(phi), z);
+}
+
+inline Float UniformSpherePdf() { return 0.25*M_1_PI; }
+
+inline point2f UniformSampleDisk(const vec2f &u) {
+  Float r = std::sqrt(u[0]);
+  Float theta = 2 * M_PI * u[1];
+  return point2f(r * std::cos(theta), r * std::sin(theta));
+}
+
+template <typename T>
+inline normal3<T> Faceforward(const normal3<T> &n, const normal3<T> &n2) {
+  return (dot(n, n2) < 0.f) ? -n : n;
+}
+
+template <typename T>
+inline vec3<T> Faceforward(const vec3<T> &v, const vec3<T> &v2) {
+  return (dot(v, v2) < 0.f) ? -v : v;
+}
+
+template <typename T>
+inline vec3<T> Faceforward(const vec3<T> &v, const normal3<T> &n2) {
+  return (dot(v, n2) < 0.f) ? -v : v;
+}
 
 #endif
