@@ -13,7 +13,12 @@ typedef float Float;
 
 #include "vec3.h"
 #include "vec2.h"
+#include "point3.h"
+#include "point2.h"
+#include "normal.h" 
 #include "mathinline.h"
+#include "transform.h"
+#include "transformcache.h"
 #include "camera.h"
 #include "float.h"
 #include "buildscene.h"
@@ -50,8 +55,6 @@ List render_scene_rcpp(List camera_info, List scene_info) {
   IntegerVector shape = as<IntegerVector>(scene_info["shape"]);
   List position_list = as<List>(scene_info["position_list"]);
   List properties = as<List>(scene_info["properties"]);
-  List velocity = as<List>(scene_info["velocity"]);
-  LogicalVector moving = as<LogicalVector>(scene_info["moving"]);
   int n = as<int>(scene_info["n"]);
   NumericVector bghigh  = as<NumericVector>(scene_info["bghigh"]);
   NumericVector bglow = as<NumericVector>(scene_info["bglow"]);
@@ -75,11 +78,7 @@ List render_scene_rcpp(List camera_info, List scene_info) {
   List order_rotation_list = as<List>(scene_info["order_rotation_list"]);
   float clampval = as<float>(scene_info["clampval"]);
   LogicalVector isgrouped = as<LogicalVector>(scene_info["isgrouped"]);
-  List group_pivot = as<List>(scene_info["group_pivot"]);
-  List group_translate = as<List>(scene_info["group_translate"]);
-  List group_angle = as<List>(scene_info["group_angle"]);
-  List group_order_rotation = as<List>(scene_info["group_order_rotation"]);
-  List group_scale = as<List>(scene_info["group_scale"]);
+  List group_transform = as<List>(scene_info["group_transform"]);
   LogicalVector tri_normal_bools = as<LogicalVector>(scene_info["tri_normal_bools"]);
   LogicalVector is_tri_color = as<LogicalVector>(scene_info["is_tri_color"]);
   List tri_color_vert = as<List>(scene_info["tri_color_vert"]);
@@ -103,6 +102,9 @@ List render_scene_rcpp(List camera_info, List scene_info) {
   List image_repeat = as<List>(scene_info["image_repeat"]);
   List csg_info = as<List>(scene_info["csg_info"]);
   List mesh_list = as<List>(scene_info["mesh_list"]);
+  List roughness_list = as<List>(scene_info["roughness_list"]);
+  List animation_info = as<List>(scene_info["animation_info"]);
+  
 
   
   auto startfirst = std::chrono::high_resolution_clock::now();
@@ -131,10 +133,10 @@ List render_scene_rcpp(List camera_info, List scene_info) {
   NumericMatrix goutput(nx,ny);
   NumericMatrix boutput(nx,ny);
   
-  vec3 lookfrom(lookfromvec[0],lookfromvec[1],lookfromvec[2]);
-  vec3 lookat(lookatvec[0],lookatvec[1],lookatvec[2]);
-  vec3 backgroundhigh(bghigh[0],bghigh[1],bghigh[2]);
-  vec3 backgroundlow(bglow[0],bglow[1],bglow[2]);
+  vec3f lookfrom(lookfromvec[0],lookfromvec[1],lookfromvec[2]);
+  vec3f lookat(lookatvec[0],lookatvec[1],lookatvec[2]);
+  vec3f backgroundhigh(bghigh[0],bghigh[1],bghigh[2]);
+  vec3f backgroundlow(bglow[0],bglow[1],bglow[2]);
   float dist_to_focus = focus_distance;
   CharacterVector alpha_files = as<CharacterVector>(alphalist["alpha_temp_file_names"]);
   LogicalVector has_alpha = as<LogicalVector>(alphalist["alpha_tex_bool"]);
@@ -143,19 +145,21 @@ List render_scene_rcpp(List camera_info, List scene_info) {
   LogicalVector has_bump = as<LogicalVector>(alphalist["bump_tex_bool"]);
   NumericVector bump_intensity = as<NumericVector>(alphalist["bump_intensity"]);
   
+  CharacterVector roughness_files = as<CharacterVector>(roughness_list["rough_temp_file_names"]);
+  LogicalVector has_roughness = as<LogicalVector>(roughness_list["rough_tex_bool"]);
   
   RcppThread::ThreadPool pool(numbercores);
   GetRNGstate();
   random_gen rng(unif_rand() * std::pow(2,32));
-  camera cam(lookfrom, lookat, vec3(camera_up(0),camera_up(1),camera_up(2)), fov, float(nx)/float(ny), 
+  camera cam(lookfrom, lookat, vec3f(camera_up(0),camera_up(1),camera_up(2)), fov, float(nx)/float(ny), 
              aperture, dist_to_focus,
              shutteropen, shutterclose);
   
-  ortho_camera ocam(lookfrom, lookat, vec3(camera_up(0),camera_up(1),camera_up(2)),
+  ortho_camera ocam(lookfrom, lookat, vec3f(camera_up(0),camera_up(1),camera_up(2)),
                     ortho_dimensions(0), ortho_dimensions(1),
                     shutteropen, shutterclose);
   
-  environment_camera ecam(lookfrom, lookat, vec3(camera_up(0),camera_up(1),camera_up(2)),
+  environment_camera ecam(lookfrom, lookat, vec3f(camera_up(0),camera_up(1),camera_up(2)),
                           shutteropen, shutterclose);
   int nx1, ny1, nn1;
   auto start = std::chrono::high_resolution_clock::now();
@@ -171,13 +175,16 @@ List render_scene_rcpp(List camera_info, List scene_info) {
   
   std::vector<Float* > bump_textures;
   std::vector<int* > nx_ny_nn_bump;
+  
+  std::vector<Float* > roughness_textures;
+  std::vector<int* > nx_ny_nn_roughness;
   //Shared material vector
   std::vector<std::shared_ptr<material> >* shared_materials = new std::vector<std::shared_ptr<material> >;
   
   for(int i = 0; i < n; i++) {
     if(isimage(i)) {
       int nx, ny, nn;
-      Float* tex_data = stbi_loadf(filelocation(i), &nx, &ny, &nn, 4);
+      Float* tex_data = stbi_loadf(filelocation(i), &nx, &ny, &nn, 0);
       textures.push_back(tex_data);
       nx_ny_nn.push_back(new int[3]);
       nx_ny_nn[i][0] = nx;
@@ -188,20 +195,22 @@ List render_scene_rcpp(List camera_info, List scene_info) {
       nx_ny_nn.push_back(nullptr);
     }
     if(has_alpha(i)) {
+      stbi_ldr_to_hdr_gamma(1.0f);
       int nxa, nya, nna;
-      Float* tex_data_alpha = stbi_loadf(alpha_files(i), &nxa, &nya, &nna, 4);
+      Float* tex_data_alpha = stbi_loadf(alpha_files(i), &nxa, &nya, &nna, 0);
       alpha_textures.push_back(tex_data_alpha);
       nx_ny_nn_alpha.push_back(new int[3]);
       nx_ny_nn_alpha[i][0] = nxa;
       nx_ny_nn_alpha[i][1] = nya;
       nx_ny_nn_alpha[i][2] = nna;
+      stbi_ldr_to_hdr_gamma(2.2f);
     } else {
       alpha_textures.push_back(nullptr);
       nx_ny_nn_alpha.push_back(nullptr);
     }
     if(has_bump(i)) {
       int nxb, nyb, nnb;
-      Float* tex_data_bump = stbi_loadf(bump_files(i), &nxb, &nyb, &nnb, 4);
+      Float* tex_data_bump = stbi_loadf(bump_files(i), &nxb, &nyb, &nnb, 0);
       bump_textures.push_back(tex_data_bump);
       nx_ny_nn_bump.push_back(new int[3]);
       nx_ny_nn_bump[i][0] = nxb;
@@ -211,11 +220,62 @@ List render_scene_rcpp(List camera_info, List scene_info) {
       bump_textures.push_back(nullptr);
       nx_ny_nn_bump.push_back(nullptr);
     }
+    if(has_roughness(i)) {
+      NumericVector temp_glossy = as<NumericVector>(glossyinfo(i));
+      int nxr, nyr, nnr;
+      Float* tex_data_roughness = stbi_loadf(roughness_files(i), &nxr, &nyr, &nnr, 0);
+      Float min = temp_glossy(9), max = temp_glossy(10);
+      Float rough_range = max-min;
+      Float maxr = 0, minr = 1;
+      for(int ii = 0; ii < nxr; ii++) {
+        for(int jj = 0; jj < nyr; jj++) {
+          Float temp_rough = tex_data_roughness[nnr*ii + nnr*nxr*jj];
+          maxr = maxr < temp_rough ? temp_rough : maxr;
+          minr = minr > temp_rough ? temp_rough : minr;
+          if(nnr > 1) {
+            temp_rough = tex_data_roughness[nnr*ii + nnr*nxr*jj+1];
+            maxr = maxr < temp_rough ? temp_rough : maxr;
+            minr = minr > temp_rough ? temp_rough : minr;
+          }
+        }
+      }
+      Float data_range = maxr-minr;
+      for(int ii = 0; ii < nxr; ii++) {
+        for(int jj = 0; jj < nyr; jj++) {
+          if(!temp_glossy(11)) {
+            tex_data_roughness[nnr*ii + nnr*nxr*jj] = 
+              (tex_data_roughness[nnr*ii + nnr*nxr*jj]-minr)/data_range * rough_range + min;
+            if(nnr > 1) {
+              tex_data_roughness[nnr*ii + nnr*nxr*jj+1] = 
+                (tex_data_roughness[nnr*ii + nnr*nxr*jj+1]-minr)/data_range * rough_range + min;
+            }
+          } else {
+            tex_data_roughness[nnr*ii + nnr*nxr*jj] = 
+              (1.0-(tex_data_roughness[nnr*ii + nnr*nxr*jj]-minr)/data_range) * rough_range + min;
+            if(nnr > 1) {
+              tex_data_roughness[nnr*ii + nnr*nxr*jj+1] = 
+                (1.0-(tex_data_roughness[nnr*ii + nnr*nxr*jj+1]-minr)/data_range) * rough_range + min;
+            }
+          }
+        }
+      }
+      roughness_textures.push_back(tex_data_roughness);
+      nx_ny_nn_roughness.push_back(new int[3]);
+      nx_ny_nn_roughness[i][0] = nxr;
+      nx_ny_nn_roughness[i][1] = nyr;
+      nx_ny_nn_roughness[i][2] = nnr;
+    } else {
+      roughness_textures.push_back(nullptr);
+      nx_ny_nn_roughness.push_back(nullptr);
+    }
   }
   
+  //Initialize transformation cache
+  TransformCache transformCache;
+  
   
   std::shared_ptr<hitable> worldbvh = build_scene(type, radius, shape, position_list,
-                                properties, velocity, moving,
+                                properties, 
                                 n,shutteropen,shutterclose,
                                 ischeckered, checkercolors, 
                                 gradient_info,
@@ -224,32 +284,34 @@ List render_scene_rcpp(List camera_info, List scene_info) {
                                 isimage, has_alpha, alpha_textures, nx_ny_nn_alpha,
                                 textures, nx_ny_nn, has_bump, bump_textures, nx_ny_nn_bump,
                                 bump_intensity,
+                                roughness_textures, nx_ny_nn_roughness, has_roughness,
                                 lightintensity, isflipped,
                                 isvolume, voldensity, order_rotation_list, 
-                                isgrouped, group_pivot, group_translate,
-                                group_angle, group_order_rotation, group_scale,
+                                isgrouped, group_transform,
                                 tri_normal_bools, is_tri_color, tri_color_vert, 
                                 fileinfo, filebasedir, 
                                 scale_list, sigmavec, glossyinfo,
                                 shared_id_mat, is_shared_mat, shared_materials,
-                                image_repeat, csg_info, mesh_list, bvh_type, rng);
+                                image_repeat, csg_info, mesh_list, bvh_type, transformCache, 
+                                animation_info, rng);
   auto finish = std::chrono::high_resolution_clock::now();
   if(verbose) {
     std::chrono::duration<double> elapsed = finish - start;
     Rcpp::Rcout << elapsed.count() << " seconds" << "\n";
   }
   
-  //Calculate world bounds
+  //Calculate world bounds and ensure camera is inside infinite area light
   aabb bounding_box_world;
   worldbvh->bounding_box(0,0,bounding_box_world);
   Float world_radius = bounding_box_world.diag.length()/2 ;
-  vec3 world_center  = bounding_box_world.centroid;
+  vec3f world_center  = bounding_box_world.centroid;
   world_radius = world_radius > (lookfrom - world_center).length() ? world_radius : (lookfrom - world_center).length()*2;
   
   if(fov == 0) {
     Float ortho_diag = sqrt(pow(ortho_dimensions(0),2) + pow(ortho_dimensions(1),2));
     world_radius += ortho_diag;
   }
+  
   //Initialize background
   if(verbose && hasbackground) {
     Rcpp::Rcout << "Loading Environment Image: ";
@@ -260,31 +322,42 @@ List render_scene_rcpp(List camera_info, List scene_info) {
   std::shared_ptr<hitable> background_sphere = nullptr;
   Float *background_texture_data = nullptr;
   
+  //Background rotation
+  Matrix4x4 Identity;
+  Transform BackgroundAngle(Identity);
+  if(rotate_env != 0) {
+    BackgroundAngle = Translate(world_center) * RotateY(rotate_env);
+  } 
+  std::shared_ptr<Transform> BackgroundTransform = transformCache.Lookup(BackgroundAngle);
+  std::shared_ptr<Transform> BackgroundTransformInv = transformCache.Lookup(BackgroundAngle.GetInverseMatrix());
+  
   if(hasbackground) {
     background_texture_data = stbi_loadf(background[0], &nx1, &ny1, &nn1, 0);
     background_texture = std::make_shared<image_texture>(background_texture_data, nx1, ny1, nn1, 1, 1, intensity_env);
     background_material = std::make_shared<diffuse_light>(background_texture, 1.0, false);
-    background_sphere = std::make_shared<InfiniteAreaLight>(nx1, ny1, world_radius*2, world_center,
-                                              background_texture, background_material);
-    if(rotate_env != 0) {
-      background_sphere = std::make_shared<rotate_y>(background_sphere, rotate_env);
-    }
+    background_sphere = std::make_shared<InfiniteAreaLight>(nx1, ny1, world_radius*2, vec3f(0.f),
+                                              background_texture, background_material, 
+                                              BackgroundTransform,
+                                              BackgroundTransformInv, false);
   } else if(ambient_light) {
     //Check if both high and low are black, and set to FLT_MIN
     if(backgroundhigh.length() == 0 && backgroundlow.length() == 0) {
-      backgroundhigh = vec3(FLT_MIN,FLT_MIN,FLT_MIN);
-      backgroundlow = vec3(FLT_MIN,FLT_MIN,FLT_MIN);
+      backgroundhigh = vec3f(FLT_MIN,FLT_MIN,FLT_MIN);
+      backgroundlow = vec3f(FLT_MIN,FLT_MIN,FLT_MIN);
     }
     background_texture = std::make_shared<gradient_texture>(backgroundlow, backgroundhigh, false, false);
     background_material = std::make_shared<diffuse_light>(background_texture, 1.0, false);
-    background_sphere = std::make_shared<InfiniteAreaLight>(100, 100, world_radius*2, world_center,
-                                              background_texture, background_material);
+    background_sphere = std::make_shared<InfiniteAreaLight>(100, 100, world_radius*2, vec3f(0.f),
+                                              background_texture, background_material,
+                                              BackgroundTransform,BackgroundTransformInv,false);
   } else {
     //Minimum intensity FLT_MIN so the CDF isn't NAN
-    background_texture = std::make_shared<constant_texture>(vec3(FLT_MIN,FLT_MIN,FLT_MIN));
+    background_texture = std::make_shared<constant_texture>(vec3f(FLT_MIN,FLT_MIN,FLT_MIN));
     background_material = std::make_shared<diffuse_light>(background_texture, 1.0, false);
-    background_sphere = std::make_shared<InfiniteAreaLight>(100, 100, world_radius*2, world_center,
-                                              background_texture, background_material);
+    background_sphere = std::make_shared<InfiniteAreaLight>(100, 100, world_radius*2, vec3f(0.f),
+                                              background_texture, background_material,
+                                              BackgroundTransform,
+                                              BackgroundTransformInv,false);
   }
   finish = std::chrono::high_resolution_clock::now();
   if(verbose && hasbackground) {
@@ -315,13 +388,13 @@ List render_scene_rcpp(List camera_info, List scene_info) {
   for(int i = 0; i < n; i++)  {
     if(implicit_sample(i)) {
       hlist.add(build_imp_sample(type, radius, shape, position_list,
-                               properties, velocity,
+                               properties, 
                                n, shutteropen, shutterclose,
                                angle, i, order_rotation_list,
-                               isgrouped, group_pivot, group_translate,
-                               group_angle, group_order_rotation, group_scale,
-                               fileinfo, filebasedir, scale_list, 
-                               mesh_list,bvh_type,  moving, rng));
+                               isgrouped, group_transform,
+                               fileinfo, filebasedir,
+                               transformCache ,scale_list, 
+                               mesh_list,bvh_type, animation_info,  rng));
     }
   }
   finish = std::chrono::high_resolution_clock::now();
@@ -387,6 +460,10 @@ List render_scene_rcpp(List camera_info, List scene_info) {
       stbi_image_free(bump_textures[i]);
       delete nx_ny_nn_bump[i];
     }
+    if(has_roughness(i)) {
+      stbi_image_free(roughness_textures[i]);
+      delete nx_ny_nn_roughness[i];
+    }
   }
   delete shared_materials;
   PutRNGstate();
