@@ -12,12 +12,12 @@ public:
   virtual ~Sampler() = 0;
   virtual void StartPixel(const unsigned int i, const unsigned int j);
   virtual Float Get1D() = 0;
-  virtual vec2 Get2D() = 0;
+  virtual vec2f Get2D() = 0;
   virtual int RoundCount(int n) const {
     return n;
   }
   const Float *Get1DArray(int n);
-  const vec2 *Get2DArray(int n);
+  const vec2f *Get2DArray(int n);
   void Request1DArray(int n);
   void Request2DArray(int n);
   
@@ -28,12 +28,12 @@ public:
   const size_t samplesPerPixel;
   
 protected:
-  //Add int vec2 class
+  //Add int vec2f class
   unsigned int currentPixelx, currentPixely;
   size_t currentPixelSampleIndex;
   std::vector<int> samples1DArraySizes, samples2DArraySizes;
   std::vector<std::vector<Float>> sampleArray1D;
-  std::vector<std::vector<vec2>> sampleArray2D;
+  std::vector<std::vector<vec2f>> sampleArray2D;
   
 private: 
   size_t array1DOffset, array2DOffset;
@@ -45,17 +45,17 @@ public:
     : Sampler(samplesPerPixel), current1DDimension(0), current2DDimension(0), rng(rng) {
     for (size_t i = 0; i < nSampledDimensions; ++i) {
       samples1D.push_back(std::vector<Float>(samplesPerPixel));
-      samples2D.push_back(std::vector<vec2>(samplesPerPixel));
+      samples2D.push_back(std::vector<vec2f>(samplesPerPixel));
     }
   }
   bool StartNextSample();
   bool SetSampleNumber(size_t);
   virtual Float Get1D();
-  virtual vec2 Get2D();
+  virtual vec2f Get2D();
   
 protected:
   std::vector<std::vector<Float> > samples1D;
-  std::vector<std::vector<vec2> > samples2D;
+  std::vector<std::vector<vec2f> > samples2D;
   size_t current1DDimension;
   size_t current2DDimension;
   random_gen rng;
@@ -80,7 +80,7 @@ public:
   RandomSampler(random_gen& rng) : PixelSampler(1 * 1, 0, rng) {}
   void StartPixel( unsigned int i,  unsigned int j) {};
   Float Get1D();
-  vec2 Get2D();
+  vec2f Get2D();
   bool StartNextSample();
   bool SetSampleNumber(size_t);
   std::unique_ptr<Sampler> Clone(int seed);
@@ -93,7 +93,7 @@ class SobolSampler : public PixelSampler {
                  random_gen& rng);
     void StartPixel( unsigned int i,  unsigned int j);
     Float Get1D();
-    vec2 Get2D();
+    vec2f Get2D();
     bool StartNextSample();
     bool SetSampleNumber(size_t);
     std::unique_ptr<Sampler> Clone(int seed);
@@ -101,8 +101,6 @@ class SobolSampler : public PixelSampler {
     const unsigned int xPixelSamples, yPixelSamples;
     unsigned int current1Dsample, current2Dsample;
     unsigned int pixelseed;
-    unsigned int pixeloffset;
-    
 };
 
 class SobolBlueNoiseSampler : public PixelSampler {
@@ -110,13 +108,12 @@ class SobolBlueNoiseSampler : public PixelSampler {
     SobolBlueNoiseSampler(random_gen& rng);
     void StartPixel( unsigned int i,  unsigned int j);
     Float Get1D();
-    vec2 Get2D();
+    vec2f Get2D();
     bool StartNextSample();
     bool SetSampleNumber(size_t);
     std::unique_ptr<Sampler> Clone(int seed);
     private:
       unsigned int current1Dsample, current2Dsample;
-      unsigned int pixelseed;
 
 };
 
