@@ -5,7 +5,9 @@
 
 
 trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, 
-        Float shutteropen, Float shutterclose, int bvh_type, random_gen rng) {
+        Float shutteropen, Float shutterclose, int bvh_type, random_gen rng,
+        std::shared_ptr<Transform> ObjectToWorld, std::shared_ptr<Transform> WorldToObject, bool reverseOrientation) : 
+  hitable(ObjectToWorld, WorldToObject, reverseOrientation) {
   tinyobj::attrib_t attrib;
   std::vector<tinyobj::shape_t > shapes;
   std::vector<tinyobj::material_t > materials;
@@ -13,6 +15,10 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
   mat_ptr = nullptr;
   
   bool ret = tinyobj::LoadObj(&attrib, &shapes, &materials, &warn, &err, inputfile.c_str(), basedir.c_str());
+  bool has_sep = true;
+  if(strlen(basedir.c_str()) == 0) {
+    has_sep = false;
+  }
   if(ret) {
     int n = 0;
     for (size_t s = 0; s < shapes.size(); s++) {
@@ -25,8 +31,8 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
     bump_materials.reserve(materials.size()+1);
     bump_textures.reserve(materials.size()+1);
     
-    std::vector<vec3 > diffuse_materials(materials.size()+1);
-    std::vector<vec3 > specular_materials(materials.size()+1);
+    std::vector<vec3f > diffuse_materials(materials.size()+1);
+    std::vector<vec3f > specular_materials(materials.size()+1);
     std::vector<Float > ior_materials(materials.size()+1);
     std::vector<bool > has_diffuse(materials.size()+1);
     std::vector<bool > has_transparency(materials.size()+1);
@@ -46,7 +52,18 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
     
     for (size_t i = 0; i < materials.size(); i++) {
       if(strlen(materials[i].diffuse_texname.c_str()) > 0) {
-        obj_materials.push_back(stbi_loadf((basedir + separator() + materials[i].diffuse_texname).c_str(), &nx, &ny, &nn, 0));
+        if(has_sep) {
+          obj_materials.push_back(stbi_loadf((basedir + separator() + materials[i].diffuse_texname).c_str(), &nx, &ny, &nn, 0));
+        } else {
+          obj_materials.push_back(stbi_loadf((materials[i].diffuse_texname).c_str(), &nx, &ny, &nn, 0));
+        }
+        if(nx == 0 || ny == 0 || nn == 0) {
+          if(has_sep) {
+            throw std::runtime_error("Could not find " + (basedir + separator() + materials[i].diffuse_texname));
+          } else {
+            throw std::runtime_error("Could not find " + materials[i].diffuse_texname);
+          }
+        }
         has_diffuse[i] = true;
         has_single_diffuse[i] = false;
         nx_mat[i] = nx;
@@ -70,7 +87,7 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
         obj_materials.push_back(nullptr);
         alpha_materials.push_back(nullptr);
         
-        diffuse_materials[i] = vec3(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
+        diffuse_materials[i] = vec3f(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
         has_diffuse[i] = true;
         has_alpha[i] = false;
         has_single_diffuse[i] = true;
@@ -86,13 +103,24 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
         obj_materials.push_back(nullptr);
         alpha_materials.push_back(nullptr);
         
-        specular_materials[i] = vec3(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
+        specular_materials[i] = vec3f(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
         ior_materials[i] = materials[i].ior;
         has_alpha[i] = false;
         has_transparency[i] = true; 
       }
       if(strlen(materials[i].bump_texname.c_str()) > 0) {
-        bump_materials[i] = stbi_loadf((basedir + separator() + materials[i].bump_texname).c_str(), &nx, &ny, &nn, 0);
+        if(has_sep) {
+          bump_materials[i] = stbi_loadf((basedir + separator() + materials[i].bump_texname).c_str(), &nx, &ny, &nn, 0);
+        } else {
+          bump_materials[i] = stbi_loadf(materials[i].bump_texname.c_str(), &nx, &ny, &nn, 0);
+        }
+        if(nx == 0 || ny == 0 || nn == 0) {
+          if(has_sep) {
+            throw std::runtime_error("Could not find " + basedir + separator() + materials[i].bump_texname);
+          } else {
+            throw std::runtime_error("Could not find " + materials[i].bump_texname);
+          }
+        }
         nx_mat_bump[i] = nx;
         ny_mat_bump[i] = ny;
         nn_mat_bump[i] = nn;
@@ -106,8 +134,8 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
     }
     bool has_normals = attrib.normals.size() > 0 ? true : false;
     
-    vec3 tris[3];
-    vec3 normals[3];
+    vec3f tris[3];
+    vec3f normals[3];
     Float tx[3];
     Float ty[3];
     for (size_t s = 0; s < shapes.size(); s++) {
@@ -120,13 +148,13 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
         for (size_t v = 0; v < 3; v++) {
           // access to vertex
           tinyobj::index_t idx = shapes[s].mesh.indices[index_offset + v];
-          tris[v] = vec3(attrib.vertices[3*idx.vertex_index+0],
+          tris[v] = vec3f(attrib.vertices[3*idx.vertex_index+0],
                          attrib.vertices[3*idx.vertex_index+1],
                          attrib.vertices[3*idx.vertex_index+2])*scale;
           
           if(has_normals && idx.normal_index != -1) {
             tempnormal = true;
-            normals[v] = vec3(attrib.normals[3*idx.normal_index+0],
+            normals[v] = vec3f(attrib.normals[3*idx.normal_index+0],
                               attrib.normals[3*idx.normal_index+1],
                               attrib.normals[3*idx.normal_index+2]);
           }
@@ -156,8 +184,8 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
           if(has_alpha[material_num]) {
             alpha = std::make_shared<alpha_texture>(obj_materials[material_num], 
                                       nx_mat[material_num], ny_mat[material_num], nn_mat[material_num], 
-                                      vec3(tx[0], tx[1], tx[2]), 
-                                      vec3(ty[0], ty[1], ty[2]));
+                                      vec3f(tx[0], tx[1], tx[2]), 
+                                      vec3f(ty[0], ty[1], ty[2]));
             alpha_materials.push_back(alpha);
           } else {
             alpha = nullptr;
@@ -165,8 +193,8 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
           if(has_bump[material_num]) {
             bump = std::make_shared<bump_texture>(bump_materials[material_num],
                                     nx_mat_bump[material_num], ny_mat_bump[material_num], nn_mat_bump[material_num],
-                                    vec3(tx[0], tx[1], tx[2]),
-                                    vec3(ty[0], ty[1], ty[2]), bump_intensity[material_num]);
+                                    vec3f(tx[0], tx[1], tx[2]),
+                                    vec3f(ty[0], ty[1], ty[2]), bump_intensity[material_num]);
             bump_textures.push_back(bump);
           } else {
             bump = nullptr;
@@ -181,11 +209,11 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
         
         if(has_normals && tempnormal) {
           if(material_num == -1) {
-            tex = std::make_shared<lambertian>(std::make_shared<constant_texture>(vec3(1,1,1)));
+            tex = std::make_shared<lambertian>(std::make_shared<constant_texture>(vec3f(1,1,1)));
           } else {
             if(has_transparency[material_num]) {
-              tex = std::make_shared<dielectric>(specular_materials[material_num], ior_materials[material_num], vec3(0,0,0), 
-                                   0,  rng);;
+              tex = std::make_shared<dielectric>(specular_materials[material_num], ior_materials[material_num], vec3f(0,0,0), 
+                                   0);;
             } else if(has_diffuse[material_num]) {
               if(has_single_diffuse[material_num]) {
                 tex = std::make_shared<lambertian>(std::make_shared<constant_texture>(diffuse_materials[material_num]));
@@ -195,19 +223,20 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
                                                                 tx[0],ty[0], tx[1],ty[1],tx[2],ty[2]));
               }
             } else {
-              tex = std::make_shared<lambertian>(std::make_shared<constant_texture>(vec3(1,1,1)));
+              tex = std::make_shared<lambertian>(std::make_shared<constant_texture>(vec3f(1,1,1)));
             }
           }
           triangles.add(std::make_shared<triangle>(tris[0],tris[1],tris[2], 
                                            normals[0], normals[1], normals[2], true, 
-                                           tex, alpha, bump));
+                                           tex, alpha, bump, 
+                                           ObjectToWorld, WorldToObject, reverseOrientation));
         } else {
           if(material_num == -1) {
             tex = std::make_shared<lambertian>(std::make_shared<constant_texture>(diffuse_materials[material_num]));
           } else {
             if(has_transparency[material_num]) {
-              tex = std::make_shared<dielectric>(specular_materials[material_num], ior_materials[material_num], vec3(0,0,0),
-                                   0,  rng);;
+              tex = std::make_shared<dielectric>(specular_materials[material_num], ior_materials[material_num], vec3f(0,0,0),
+                                   0);;
             } else if(has_diffuse[material_num]) {
               if(has_single_diffuse[material_num]) {
                 tex = std::make_shared<lambertian>(std::make_shared<constant_texture>(diffuse_materials[material_num]));
@@ -219,10 +248,11 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
                                                                 tx[2],ty[2]));
               }
             } else {
-              tex = std::make_shared<lambertian>(std::make_shared<constant_texture>(vec3(1,1,1)));
+              tex = std::make_shared<lambertian>(std::make_shared<constant_texture>(vec3f(1,1,1)));
             }
           }
-          triangles.add(std::make_shared<triangle>(tris[0],tris[1],tris[2], true, tex, alpha, bump));
+          triangles.add(std::make_shared<triangle>(tris[0],tris[1],tris[2], true, tex, alpha, bump, 
+                                                   ObjectToWorld, WorldToObject, reverseOrientation));
         }
       }
     }
@@ -234,7 +264,9 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale,
 }
 
 trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float sigma,
-        Float shutteropen, Float shutterclose, int bvh_type, random_gen rng) {
+        Float shutteropen, Float shutterclose, int bvh_type, random_gen rng,
+        std::shared_ptr<Transform> ObjectToWorld, std::shared_ptr<Transform> WorldToObject, bool reverseOrientation) :
+      hitable(ObjectToWorld, WorldToObject, reverseOrientation) {
   tinyobj::attrib_t attrib;
   std::vector<tinyobj::shape_t > shapes;
   std::vector<tinyobj::material_t > materials;
@@ -242,7 +274,10 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
   mat_ptr = nullptr;
   
   bool ret = tinyobj::LoadObj(&attrib, &shapes, &materials, &warn, &err, inputfile.c_str(), basedir.c_str());
-  
+  bool has_sep = true;
+  if(strlen(basedir.c_str()) == 0) {
+    has_sep = false;
+  }
   if(ret) {
     int n = 0;
     for (size_t s = 0; s < shapes.size(); s++) {
@@ -254,8 +289,8 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
     obj_materials.reserve(materials.size()+1);
     bump_materials.reserve(materials.size()+1);
     
-    std::vector<vec3 > diffuse_materials(materials.size()+1);
-    std::vector<vec3 > specular_materials(materials.size()+1);
+    std::vector<vec3f > diffuse_materials(materials.size()+1);
+    std::vector<vec3f > specular_materials(materials.size()+1);
     std::vector<Float > ior_materials(materials.size()+1);
     std::vector<bool > has_diffuse(materials.size()+1);
     std::vector<bool > has_transparency(materials.size()+1);
@@ -276,7 +311,18 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
     
     for (size_t i = 0; i < materials.size(); i++) {
       if(strlen(materials[i].diffuse_texname.c_str()) > 0) {
-        obj_materials.push_back(stbi_loadf((basedir + separator() + materials[i].diffuse_texname).c_str(), &nx, &ny, &nn, 0));
+        if(has_sep) {
+          obj_materials.push_back(stbi_loadf((basedir + separator() + materials[i].diffuse_texname).c_str(), &nx, &ny, &nn, 0));
+        } else {
+          obj_materials.push_back(stbi_loadf((materials[i].diffuse_texname).c_str(), &nx, &ny, &nn, 0));
+        }
+        if(nx == 0 || ny == 0 || nn == 0) {
+          if(has_sep) {
+            throw std::runtime_error("Could not find " + (basedir + separator() + materials[i].diffuse_texname));
+          } else {
+            throw std::runtime_error("Could not find " + materials[i].diffuse_texname);
+          }
+        }
         has_diffuse[i] = true;
         has_single_diffuse[i] = false;
         nx_mat[i] = nx;
@@ -300,7 +346,7 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
         obj_materials.push_back(nullptr);
         alpha_materials.push_back(nullptr);
         
-        diffuse_materials[i] = vec3(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
+        diffuse_materials[i] = vec3f(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
         has_diffuse[i] = true;
         has_single_diffuse[i] = true;
       } else {
@@ -314,12 +360,25 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
         obj_materials.push_back(nullptr);
         alpha_materials.push_back(nullptr);
         
-        specular_materials[i] = vec3(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
+        specular_materials[i] = vec3f(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
         ior_materials[i] = materials[i].ior;
         has_transparency[i] = true; 
       }
       if(strlen(materials[i].bump_texname.c_str()) > 0) {
         bump_materials[i] = stbi_loadf((basedir + separator() + materials[i].bump_texname).c_str(), &nx, &ny, &nn, 0);
+        if(has_sep) {
+          bump_materials[i] = stbi_loadf((basedir + separator() + materials[i].bump_texname).c_str(), &nx, &ny, &nn, 0);
+        } else {
+          bump_materials[i] = stbi_loadf(materials[i].bump_texname.c_str(), &nx, &ny, &nn, 0);
+        }
+        
+        if(nx == 0 || ny == 0 || nn == 0) {
+          if(has_sep) {
+            throw std::runtime_error("Could not find " + basedir + separator() + materials[i].bump_texname);
+          } else {
+            throw std::runtime_error("Could not find " + materials[i].bump_texname);
+          }
+        }
         nx_mat_bump[i] = nx;
         ny_mat_bump[i] = ny;
         nn_mat_bump[i] = nn;
@@ -332,8 +391,8 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
       }
     }
     bool has_normals = attrib.normals.size() > 0 ? true : false;
-    vec3 tris[3];
-    vec3 normals[3];
+    vec3f tris[3];
+    vec3f normals[3];
     Float tx[3];
     Float ty[3];
     for (size_t s = 0; s < shapes.size(); s++) {
@@ -346,12 +405,12 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
         for (size_t v = 0; v < 3; v++) {
           // access to vertex
           tinyobj::index_t idx = shapes[s].mesh.indices[index_offset + v];
-          tris[v] = vec3(attrib.vertices[3*idx.vertex_index+0],
+          tris[v] = vec3f(attrib.vertices[3*idx.vertex_index+0],
                          attrib.vertices[3*idx.vertex_index+1],
                                         attrib.vertices[3*idx.vertex_index+2])*scale;
           if(has_normals && idx.normal_index != -1) {
             tempnormal = true;
-            normals[v] = vec3(attrib.normals[3*idx.normal_index+0],
+            normals[v] = vec3f(attrib.normals[3*idx.normal_index+0],
                               attrib.normals[3*idx.normal_index+1],
                                             attrib.normals[3*idx.normal_index+2]);
           }
@@ -374,8 +433,8 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
         if(material_num > -1) {
           if(has_alpha[material_num]) {
             alpha = std::make_shared<alpha_texture>(obj_materials[material_num], nx, ny, nn, 
-                                      vec3(tx[0], tx[1], tx[2]), 
-                                      vec3(ty[0], ty[1], ty[2]));
+                                      vec3f(tx[0], tx[1], tx[2]), 
+                                      vec3f(ty[0], ty[1], ty[2]));
             alpha_materials.push_back(alpha);
           } else {
             alpha = nullptr;
@@ -384,8 +443,8 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
           if(has_bump[material_num]) {
             bump = std::make_shared<bump_texture>(bump_materials[material_num],
                                     nx_mat_bump[material_num], ny_mat_bump[material_num], nn_mat_bump[material_num],
-                                                                                                     vec3(tx[0], tx[1], tx[2]),
-                                                                                                     vec3(ty[0], ty[1], ty[2]), bump_intensity[material_num]);
+                                                                                                     vec3f(tx[0], tx[1], tx[2]),
+                                                                                                     vec3f(ty[0], ty[1], ty[2]), bump_intensity[material_num]);
             bump_textures.push_back(bump);
           } else {
             bump = nullptr;
@@ -397,33 +456,34 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
         
         if(has_normals && tempnormal) {
           if(material_num == -1) {
-            tex = std::make_shared<orennayar>(std::make_shared<constant_texture>(vec3(1,1,1)), sigma);
+            tex = std::make_shared<orennayar>(std::make_shared<constant_texture>(vec3f(1,1,1)), sigma);
           } else {
             if(has_transparency[material_num]) {
-              tex = std::make_shared<dielectric>(specular_materials[material_num], ior_materials[material_num], vec3(0,0,0), 
-                                   0,  rng);;
+              tex = std::make_shared<dielectric>(specular_materials[material_num], ior_materials[material_num], vec3f(0,0,0), 
+                                   0);;
             } else if(has_diffuse[material_num]) {
               if(has_single_diffuse[material_num]) {
                 tex = std::make_shared<orennayar>(std::make_shared<constant_texture>(diffuse_materials[material_num]), sigma);
               } else {
                 tex = std::make_shared<orennayar>(std::make_shared<triangle_image_texture>(obj_materials[material_num],
                                                                nx_mat[material_num], ny_mat[material_num],nn_mat[material_num],
-                                                                                                                tx[0],ty[0], tx[1],ty[1],tx[2],ty[2]), sigma);
+                                                               tx[0],ty[0], tx[1],ty[1],tx[2],ty[2]), sigma);
               }
             } else {
-              tex = std::make_shared<orennayar>(std::make_shared<constant_texture>(vec3(1,1,1)), sigma);
+              tex = std::make_shared<orennayar>(std::make_shared<constant_texture>(vec3f(1,1,1)), sigma);
             }
           }
           triangles.add(std::make_shared<triangle>(tris[0],tris[1],tris[2], 
                                            normals[0], normals[1], normals[2], true, 
-                                           tex, alpha,  bump));
+                                           tex, alpha,  bump, 
+                                           ObjectToWorld, WorldToObject, reverseOrientation));
         } else {
           if(material_num == -1) {
             tex = std::make_shared<orennayar>(std::make_shared<constant_texture>(diffuse_materials[material_num]), sigma);
           } else {
             if(has_transparency[material_num]) {
-              tex = std::make_shared<dielectric>(specular_materials[material_num], ior_materials[material_num], vec3(0,0,0), 
-                                   0,  rng);;
+              tex = std::make_shared<dielectric>(specular_materials[material_num], ior_materials[material_num], vec3f(0,0,0), 
+                                   0);;
             } else if(has_diffuse[material_num]) {
               if(has_single_diffuse[material_num]) {
                 tex = std::make_shared<orennayar>(std::make_shared<constant_texture>(diffuse_materials[material_num]), sigma);
@@ -433,10 +493,11 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
                                                                                                                tx[0],ty[0],tx[1],ty[1],tx[2],ty[2]), sigma);
               }
             } else {
-              tex = std::make_shared<orennayar>(std::make_shared<constant_texture>(vec3(1,1,1)), sigma);
+              tex = std::make_shared<orennayar>(std::make_shared<constant_texture>(vec3f(1,1,1)), sigma);
             }
           }
-          triangles.add(std::make_shared<triangle>(tris[0],tris[1],tris[2], true, tex, alpha, bump));
+          triangles.add(std::make_shared<triangle>(tris[0],tris[1],tris[2], true, tex, alpha, bump, 
+                                                   ObjectToWorld, WorldToObject, reverseOrientation));
         }
       }
     }
@@ -448,7 +509,9 @@ trimesh::trimesh(std::string inputfile, std::string basedir, Float scale, Float
 }
 
 trimesh::trimesh(std::string inputfile, std::string basedir, std::shared_ptr<material> mat, 
-        Float scale, Float shutteropen, Float shutterclose, int bvh_type, random_gen rng) {
+        Float scale, Float shutteropen, Float shutterclose, int bvh_type, random_gen rng,
+        std::shared_ptr<Transform> ObjectToWorld, std::shared_ptr<Transform> WorldToObject, bool reverseOrientation) :
+    hitable(ObjectToWorld, WorldToObject, reverseOrientation) {
   tinyobj::attrib_t attrib;
   std::vector<tinyobj::shape_t > shapes;
   std::vector<tinyobj::material_t > materials;
@@ -456,6 +519,10 @@ trimesh::trimesh(std::string inputfile, std::string basedir, std::shared_ptr<mat
   mat_ptr = mat;
   
   bool ret = tinyobj::LoadObj(&attrib, &shapes, &materials, &warn, &err, inputfile.c_str(), basedir.c_str());
+  bool has_sep = true;
+  if(strlen(basedir.c_str()) == 0) {
+    has_sep = false;
+  }
   std::shared_ptr<alpha_texture> alpha = nullptr;
   std::shared_ptr<bump_texture> bump = nullptr;
   if(ret) {
@@ -464,8 +531,8 @@ trimesh::trimesh(std::string inputfile, std::string basedir, std::shared_ptr<mat
       n += shapes[s].mesh.num_face_vertices.size();
     }
     bool has_normals = attrib.normals.size() > 0 ? true : false;
-    vec3 tris[3];
-    vec3 normals[3];
+    vec3f tris[3];
+    vec3f normals[3];
     for (size_t s = 0; s < shapes.size(); s++) {
       
       size_t index_offset = 0;
@@ -475,14 +542,14 @@ trimesh::trimesh(std::string inputfile, std::string basedir, std::shared_ptr<mat
         for (size_t v = 0; v < 3; v++) {
           tinyobj::index_t idx = shapes[s].mesh.indices[index_offset + v];
           
-          tris[v] = vec3(attrib.vertices[3*idx.vertex_index+0],
+          tris[v] = vec3f(attrib.vertices[3*idx.vertex_index+0],
                          attrib.vertices[3*idx.vertex_index+1],
                                         attrib.vertices[3*idx.vertex_index+2])*scale;
           
           if(has_normals  && idx.normal_index != -1) {
             tempnormal = true;
             
-            normals[v] = vec3(attrib.normals[3*idx.normal_index+0],
+            normals[v] = vec3f(attrib.normals[3*idx.normal_index+0],
                               attrib.normals[3*idx.normal_index+1],
                                             attrib.normals[3*idx.normal_index+2]);
           }
@@ -506,10 +573,12 @@ trimesh::trimesh(std::string inputfile, std::string basedir, std::shared_ptr<mat
           triangles.add(std::make_shared<triangle>(tris[0],tris[1],tris[2],
                                            normals[0],normals[1],normals[2], 
                                            false,
-                                           mat_ptr, alpha,  bump));
+                                           mat_ptr, alpha,  bump, 
+                                           ObjectToWorld, WorldToObject, reverseOrientation));
         } else {
           triangles.add(std::make_shared<triangle>(tris[0],tris[1],tris[2], false, 
-                                                   mat_ptr, alpha, bump));
+                                                   mat_ptr, alpha, bump, 
+                                                   ObjectToWorld, WorldToObject, reverseOrientation));
         }
       }
     }
@@ -522,7 +591,8 @@ trimesh::trimesh(std::string inputfile, std::string basedir, std::shared_ptr<mat
 
 trimesh::trimesh(std::string inputfile, std::string basedir, float vertex_color_sigma,
         Float scale, bool is_vertex_color, Float shutteropen, Float shutterclose, int bvh_type, 
-        random_gen rng) {
+        random_gen rng,
+        std::shared_ptr<Transform> ObjectToWorld, std::shared_ptr<Transform> WorldToObject, bool reverseOrientation) : hitable(ObjectToWorld, WorldToObject, reverseOrientation) {
   tinyobj::attrib_t attrib;
   std::vector<tinyobj::shape_t > shapes;
   std::vector<tinyobj::material_t > materials;
@@ -534,6 +604,10 @@ trimesh::trimesh(std::string inputfile, std::string basedir, float vertex_color_
   std::shared_ptr<bump_texture> bump = nullptr;
   
   bool ret = tinyobj::LoadObj(&attrib, &shapes, &materials, &warn, &err, inputfile.c_str(), basedir.c_str());
+  bool has_sep = true;
+  if(strlen(basedir.c_str()) == 0) {
+    has_sep = false;
+  }
   if(ret) {
     int n = 0;
     for (size_t s = 0; s < shapes.size(); s++) {
@@ -541,30 +615,30 @@ trimesh::trimesh(std::string inputfile, std::string basedir, float vertex_color_
     }
     bool has_normals = attrib.normals.size() > 0 ? true : false;
     bool has_vertex_colors = attrib.colors.size() > 0 ? true : false;
-    vec3 tris[3];
-    vec3 normals[3];
-    vec3 colors[3];
+    vec3f tris[3];
+    vec3f normals[3];
+    vec3f colors[3];
     for (size_t s = 0; s < shapes.size(); s++) {
       size_t index_offset = 0;
       for (size_t f = 0; f < shapes[s].mesh.num_face_vertices.size(); f++) {
         bool tempnormal = false;
         
-        colors[0] = vec3(1,1,1);
-        colors[1] = vec3(1,1,1);
-        colors[2] = vec3(1,1,1);
+        colors[0] = vec3f(1,1,1);
+        colors[1] = vec3f(1,1,1);
+        colors[2] = vec3f(1,1,1);
         for (size_t v = 0; v < 3; v++) {
           tinyobj::index_t idx = shapes[s].mesh.indices[index_offset + v];
           
-          tris[v] = vec3(attrib.vertices[3*idx.vertex_index+0],
+          tris[v] = vec3f(attrib.vertices[3*idx.vertex_index+0],
                          attrib.vertices[3*idx.vertex_index+1],
                                         attrib.vertices[3*idx.vertex_index+2])*scale;
           if(has_vertex_colors) {
-            colors[v] = vec3(attrib.colors[3*idx.vertex_index+0],
+            colors[v] = vec3f(attrib.colors[3*idx.vertex_index+0],
                              attrib.colors[3*idx.vertex_index+1],
                                           attrib.colors[3*idx.vertex_index+2]);
           }
           if(has_normals && idx.normal_index != -1) {
-            normals[v] = vec3(attrib.normals[3*idx.normal_index+0],
+            normals[v] = vec3f(attrib.normals[3*idx.normal_index+0],
                               attrib.normals[3*idx.normal_index+1],
                                             attrib.normals[3*idx.normal_index+2]);
           }
@@ -590,9 +664,11 @@ trimesh::trimesh(std::string inputfile, std::string basedir, float vertex_color_
           triangles.add(std::make_shared<triangle>(tris[0],tris[1],tris[2],
                                            normals[0],normals[1],normals[2], 
                                           true,
-                                          tex, alpha, bump));
+                                          tex, alpha, bump, 
+                                          ObjectToWorld, WorldToObject, reverseOrientation));
         } else {
-          triangles.add(std::make_shared<triangle>(tris[0],tris[1],tris[2], true, tex, alpha, bump));
+          triangles.add(std::make_shared<triangle>(tris[0],tris[1],tris[2], true, tex, alpha, bump, 
+                                                   ObjectToWorld, WorldToObject, reverseOrientation));
         }
       }
     }
@@ -616,20 +692,20 @@ bool trimesh::bounding_box(Float t0, Float t1, aabb& box) const {
 }
 
 
-Float trimesh::pdf_value(const vec3& o, const vec3& v, random_gen& rng, Float time) {
+Float trimesh::pdf_value(const point3f& o, const vec3f& v, random_gen& rng, Float time) {
   return(triangles.pdf_value(o,v, rng, time));
 }
 
-Float trimesh::pdf_value(const vec3& o, const vec3& v, Sampler* sampler, Float time) {
+Float trimesh::pdf_value(const point3f& o, const vec3f& v, Sampler* sampler, Float time) {
   return(triangles.pdf_value(o,v, sampler, time));
   
 }
 
-vec3 trimesh::random(const vec3& o, random_gen& rng, Float time) {
+vec3f trimesh::random(const point3f& o, random_gen& rng, Float time) {
   return(triangles.random(o, rng, time));
 }
 
-vec3 trimesh::random(const vec3& o, Sampler* sampler, Float time) {
+vec3f trimesh::random(const point3f& o, Sampler* sampler, Float time) {
   return(triangles.random(o, sampler, time));
   
 }
